import pytest

from pandevice import device
from tests.live import testlib


class TestDeviceConfig(object):
    def toggle_object_variable(self, obj, var, new_value):
        original_value = getattr(obj, var)
        for value in (new_value, original_value):
            setattr(obj, var, value)
            obj.update(var)

    def test_01_get_device_config(self, dev, state_map):
        state = state_map.setdefault(dev)
        state.got_device_config = False

        dco = device.SystemSettings.refreshall(dev)
        state.got_device_config = True
        state.dco = dco[0]

    def test_02_update_hostname(self, dev, state_map):
        state = state_map.setdefault(dev)
        if not state.got_device_config:
            pytest.xfail('failed to get device config')

        # Change the hostname
        self.toggle_object_variable(state.dco, 'hostname',
                                    testlib.random_name())

    def test_03_update_secondary_dns(self, dev, state_map):
        state = state_map.setdefault(dev)
        if not state.got_device_config:
            pytest.xfail('failed to get device config')

        # Toggle the secondary ip address
        self.toggle_object_variable(state.dco, 'dns_secondary',
                                    testlib.random_ip())

    def test_04_create_ntp(self, dev, state_map):
        state = state_map.setdefault(dev)
        if not state.got_device_config:
            pytest.xfail('failed to get device config')

        primary = None
        secondary = None

        for x in state.dco.children:
            if x.__class__ == device.NTPServerPrimary:
                primary = x
            elif x.__class__ == device.NTPServerSecondary:
                secondary = x

        state.restore_ntp = False
        if primary is None:
            state.ntp_obj = device.NTPServerPrimary(
                address=testlib.random_ip())
        elif secondary is None:
            state.ntp_obj = device.NTPServerSecondary(
                address=testlib.random_ip())
        else:
            state.created_ntp = True
            state.restore_ntp = True
            state.ntp_obj = secondary
            pytest.skip('Both primary and secondary exist, nothing to create')

        state.dco.add(state.ntp_obj)
        state.ntp_obj.create()
        state.created_ntp = True

    def test_05_update_ntp(self, dev, state_map):
        state = state_map.setdefault(dev)
        if not state.got_device_config:
            pytest.xfail('failed to get device config')
        if not state.created_ntp:
            pytest.xfail('failed to create ntp in previous step')

        self.toggle_object_variable(state.ntp_obj, 'address',
                                    testlib.random_ip())

    def test_06_delete_ntp(self, dev, state_map):
        state = state_map.setdefault(dev)
        if not state.got_device_config:
            pytest.xfail('failed to get device config')
        if not state.created_ntp:
            pytest.xfail('failed to create ntp in previous step')

        state.ntp_obj.delete()

    def test_07_restore_ntp(self, dev, state_map):
        state = state_map.setdefault(dev)
        if not state.got_device_config:
            pytest.xfail('failed to get device config')
        if not state.restore_ntp:
            pytest.skip('restore not needed')

        state.dco.add(state.ntp_obj)
        state.ntp_obj.create()

    def test_99_removeall(self, dev, state_map):
        dev.removeall()


class TestPasswordProfile(testlib.DevFlow):
    def setup_state_obj(self, dev, state):
        state.obj = device.PasswordProfile(
            testlib.random_name(), 0, 0, 0, 0)
        dev.add(state.obj)

    def update_state_obj(self, dev, state):
        state.obj.expiration = 120
        state.obj.warning = 15
        state.obj.login_count = 1
        state.obj.grace_period = 15


class TestFirewallAdministrator(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.profiles = []
        for x in range(2):
            state.profiles.append(device.PasswordProfile(
                testlib.random_name(), x, x, x, x))
            fw.add(state.profiles[x])

        state.profiles[0].create_similar()

    def setup_state_obj(self, fw, state):
        state.obj = device.Administrator(
            testlib.random_name(),
            superuser=True,
            password_profile=state.profiles[0])
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.password_profile = state.profiles[1]

    def test_05_superuser_read_only(self, fw, state_map):
        state = self.sanity(fw, state_map)

        state.obj.superuser = None
        state.obj.superuser_read_only = True

        state.obj.apply()

    def test_06_device_admin(self, fw, state_map):
        state = self.sanity(fw, state_map)

        state.obj.superuser_read_only = None
        state.obj.device_admin = True

        state.obj.apply()

    def test_07_device_admin_read_only(self, fw, state_map):
        state = self.sanity(fw, state_map)

        state.obj.device_admin = None
        state.obj.device_admin_read_only = True

        state.obj.apply()

    def test_08_set_password(self, fw, state_map):
        state = self.sanity(fw, state_map)

        # Set the password
        state.obj.change_password('secret')

        # Now verify the change by trying to login
        new_fw = fw.__class__(
            fw.hostname, state.obj.uid, 'secret')
        new_fw.refresh_system_info()

    def cleanup_dependencies(self, fw, state):
        try:
            state.profiles[0].delete_similar()
        except IndexError:
            pass


class TestPanoramaAdministrator(testlib.PanoFlow):
    def create_dependencies(self, pano, state):
        state.profiles = []
        for x in range(2):
            state.profiles.append(device.PasswordProfile(
                testlib.random_name(), x, x, x, x))
            pano.add(state.profiles[x])

        state.profiles[0].create_similar()

    def setup_state_obj(self, pano, state):
        state.obj = device.Administrator(
            testlib.random_name(),
            superuser=True,
            password_profile=state.profiles[0])
        pano.add(state.obj)

    def update_state_obj(self, pano, state):
        state.obj.password_profile = state.profiles[1]

    def test_05_superuser_read_only(self, pano, state_map):
        state = self.sanity(pano, state_map)

        state.obj.superuser = None
        state.obj.superuser_read_only = True

        state.obj.apply()

    def test_06_panorama_admin(self, pano, state_map):
        state = self.sanity(pano, state_map)

        state.obj.superuser_read_only = None
        state.obj.panorama_admin = True

        state.obj.apply()

    def test_07_set_password(self, pano, state_map):
        state = self.sanity(pano, state_map)

        # Set the password
        state.obj.change_password('secret')

        # Now verify the change by trying to login
        new_pano = pano.__class__(
            pano.hostname, state.obj.uid, 'secret')
        new_pano.refresh_system_info()

    def cleanup_dependencies(self, pano, state):
        try:
            state.profiles[0].delete_similar()
        except IndexError:
            pass



#!/usr/bin/env python

# Copyright (c) 2014, Palo Alto Networks
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


"""Policies module contains policies and rules that exist in the 'Policies' tab in the firewall GUI"""

# import modules
from pandevice import getlogger
from pandevice.base import PanObject, Root, MEMBER, ENTRY
from pandevice.base import VarPath as Var
from pandevice.base import VersionedPanObject
from pandevice.base import VersionedParamPath

# import other parts of this pandevice package
import pandevice.errors as err

logger = getlogger(__name__)


class Rulebase(VersionedPanObject):
    """Rulebase for a Firewall

    Firewall only.  For Panorama, use :class:`pandevice.policies.PreRulebase` or
    :class:`pandevice.policies.PostRulebase`.

    """
    ROOT = Root.VSYS
    CHILDTYPES = (
        "policies.SecurityRule",
        "policies.NatRule",
    )

    def _setup(self):
        self._xpaths.add_profile(value='/rulebase')


class PreRulebase(Rulebase):
    """Pre-rulebase for a Panorama

    Panorama only.  For Firewall, use :class:`pandevice.policies.Rulebase`.

    """
    def _setup(self):
        self._xpaths.add_profile(value='/pre-rulebase')


class PostRulebase(Rulebase):
    """Post-rulebase for a Panorama

    Panorama only.  For Firewall, use :class:`pandevice.policies.Rulebase`.

    """
    def _setup(self):
        self._xpaths.add_profile(value='/post-rulebase')


class SecurityRule(VersionedPanObject):
    """Security Rule

    Firewall only.  Use SecurityPreRule or SecurityPostRule with Panorama.

    Args:
        name (str): Name of the rule
        fromzone (list): From zones
        tozone (list): To zones
        source (list): Source addresses
        destination (list): Destination addresses
        application (list): Applications
        service (list): Destination services (ports) (Default:
            application-default)
        category (list): Destination URL Categories
        action (str): Action to take (deny, allow, drop, reset-client,
            reset-server, reset-both)
            Note: Not all options are available on all PAN-OS versions.
        log_setting (str): Log forwarding profile
        log_start (bool): Log at session start
        log_end (bool): Log at session end
        description (str): Description of this rule
        type (str): 'universal', 'intrazone', or 'intrazone' (Default:
            universal)
        tag (list): Administrative tags
        negate_source (bool): Match on the reverse of the 'source' attribute
        negate_destination (bool): Match on the reverse of the 'destination'
            attribute
        disabled (bool): Disable this rule
        schedule (str): Schedule Profile
        icmp_unreachable (bool): Send ICMP Unreachable
        disable_server_response_inspection (bool): Disable server response
            inspection
        group (str): Security Profile Group
        virus (str): Antivirus Security Profile
        spyware (str): Anti-Spyware Security Profile
        vulnerability (str): Vulnerability Protection Security Profile
        url_filtering (str): URL Filtering Security Profile
        file_blocking (str): File Blocking Security Profile
        wildfire_analysis (str): Wildfire Analysis Security Profile
        data_filtering (str): Data Filtering Security Profile
        negate_target (bool): Target all but the listed target firewalls
            (applies to panorama/device groups only)
        target (list): Apply this policy to the listed firewalls only
            (applies to panorama/device groups only)

    """
    # TODO: Add QoS variables
    SUFFIX = ENTRY

    def _setup(self):
        # xpaths
        self._xpaths.add_profile(value='/security/rules')

        # params
        params = []

        any_defaults = (
            ('fromzone', 'from'), ('tozone', 'to'), ('source', 'source'),
            ('source_user', 'source-user'), ('hip_profiles', 'hip-profiles'),
            ('destination', 'destination'), ('application', 'application'),
        )
        for var_name, path in any_defaults:
            params.append(VersionedParamPath(
                var_name, default=['any', ], vartype='member', path=path))

        params.append(VersionedParamPath(
            'service', default='application-default',
            vartype='member', path='service'))
        params.append(VersionedParamPath(
            'category', default=['any', ], vartype='member', path='category'))
        params.append(VersionedParamPath(
            'action', path='action'))
        params.append(VersionedParamPath(
            'log_setting', path='log-setting'))
        params.append(VersionedParamPath(
            'log_start', path='log-start', vartype='yesno'))
        params.append(VersionedParamPath(
            'log_end', path='log-end', vartype='yesno'))
        params.append(VersionedParamPath(
            'description', path='description'))
        params.append(VersionedParamPath(
            'type', default='universal', path='rule-type'))
        params.append(VersionedParamPath(
            'tag', path='tag', vartype='member'))
        params.append(VersionedParamPath(
            'negate_source', path='negate-source', vartype='yesno'))
        params.append(VersionedParamPath(
            'negate_destination', path='negate-destination', vartype='yesno'))
        params.append(VersionedParamPath(
            'disabled', path='disabled', vartype='yesno'))
        params.append(VersionedParamPath(
            'schedule', path='schedule'))
        params.append(VersionedParamPath(
            'icmp_unreachable', path='icmp-unreachable'))
        params.append(VersionedParamPath(
            'disable_server_response_inspection', vartype='yesno',
            path='option/disable-server-response-inspection'))
        params.append(VersionedParamPath(
            'group', path='profile-setting/group', vartype='member'))
        params.append(VersionedParamPath(
            'negate_target', path='target/negate', vartype='yesno'))
        params.append(VersionedParamPath(
            'target', path='target/devices', vartype='entry'))

        member_profiles = (
            'virus', 'spyware', 'vulnerability', 'url-filtering',
            'file-blocking', 'wildfire-analysis', 'data-filtering',
        )
        for p in member_profiles:
            params.append(VersionedParamPath(
                p, vartype='member',
                path='profile-setting/profiles/{0}'.format(p)))

        self._params = tuple(params)


class NatRule(VersionedPanObject):
    """NAT Rule

    Both the naming convention and the order of the parameters tries to closly
    match what is presented in the GUI.

    There are groupings of parameters that give hints to the sections that
    they contribute towards:

        * source_translation_<etc>
        * source_translation_fallback_<etc>
        * source_translation_static_<etc>
        * destination_translation_<etc>

    Args:
        name (str): Name of the rule
        description (str): The description
        nat_type (str): Type of NAT
        fromzone (list): From zones
        tozone (list): To zones
        to_interface (str): Egress interface from route lookup
        service (str): The service
        source (list): Source addresses
        destination (list): Destination addresses
        source_translation_type (str): Type of source address translation
        source_translation_address_type (str): Address type for Dynamic IP
            And Port or Dynamic IP source translation types
        source_translation_interface (str): Interface of the source address
            translation for Dynamic IP and Port source translation types
        source_translation_ip_address (str): IP address of the source address
            translation for Dynamic IP and Port source translation types
        source_translation_translated_addresses (list): Translated addresses
            of the source address translation for Dynamic IP And Port or
            Dynamic IP source translation types
        source_translation_fallback_type (str): Type of fallback for Dynamic IP
            source translation types
        source_translation_fallback_translated_addresses (list): Addresses for
            translated address types of fallback source translation
        source_translation_fallback_interface (str): The interface for the
            fallback source translation
        source_translation_fallback_ip_type (str): The type of the IP address
            for the fallback source translation IP address
        source_translation_fallback_ip_address (str): The IP address of the
            fallback source translation
        source_translation_static_translated_address (str): The IP address
            for the static source translation
        source_translation_static_bi_directional (bool): Allow reverse
            translation from translated address to original address
        destination_translated_address (str): Translated destination IP
            address
        destination_translated_port (int): Translated destination port number
        ha_binding (str): Device binding configuration in HA Active-Active mode
        disabled (bool): Disable this rule
        negate_target (bool): Target all but the listed target firewalls
            (applies to panorama/device groups only)
        target (list): Apply this policy to the listed firewalls only
            (applies to panorama/device groups only)
        tag (list): Administrative tags
    """
    SUFFIX = ENTRY

    def _setup(self):
        # xpaths
        self._xpaths.add_profile(value='/nat/rules')

        # params
        params = []

        params.append(VersionedParamPath(
            'description', path='description'))
        params.append(VersionedParamPath(
            'nat_type', path='nat-type', default='ipv4',
            values=('ipv4', 'nat64', 'nptv6')))
        params.append(VersionedParamPath(
            'fromzone', default=['any', ], vartype='member', path='from'))
        params.append(VersionedParamPath(
            'tozone', vartype='member', path='to'))
        params.append(VersionedParamPath(
            'to_interface', path='to-interface'))
        params.append(VersionedParamPath(
            'service', default='any', path='service'))
        params.append(VersionedParamPath(
            'source', default=['any', ], vartype='member', path='source'))
        params.append(VersionedParamPath(
            'destination', default=['any', ],
            vartype='member', path='destination'))
        params.append(VersionedParamPath(
            'source_translation_type',
            path='source-translation/{source_translation_type}',
            values=(
                'dynamic-ip-and-port',
                'dynamic-ip',
                'static-ip')))
        params.append(VersionedParamPath(
            'source_translation_address_type',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                '{source_translation_address_type}')),
            values=('interface-address', 'translated-address'),
            default='translated-address',
            condition={
                'source_translation_type': [
                    'dynamic-ip-and-port', 'dynamic-ip']}))
        params.append(VersionedParamPath(
            'source_translation_interface',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                '{source_translation_address_type}',
                'interface')),
            condition={
                'source_translation_type': 'dynamic-ip-and-port',
                'source_translation_address_type': 'interface-address'}))
        params.append(VersionedParamPath(
            'source_translation_ip_address',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                '{source_translation_address_type}',
                'ip')),
            condition={
                'source_translation_type': 'dynamic-ip-and-port',
                'source_translation_address_type': 'interface-address'}))
        params.append(VersionedParamPath(
            'source_translation_translated_addresses',
            vartype='member',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                '{source_translation_address_type}')),
            condition={
                'source_translation_type': [
                    'dynamic-ip-and-port', 'dynamic-ip'],
                'source_translation_address_type': 'translated-address'}))
        params.append(VersionedParamPath(
            'source_translation_fallback_type',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                'fallback',
                '{source_translation_fallback_type}')),
            values=('translated-address', 'interface-address'),
            condition={
                'source_translation_type': 'dynamic-ip'}))
        params.append(VersionedParamPath(
            'source_translation_fallback_translated_addresses',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                'fallback',
                '{source_translation_fallback_type}')),
            vartype='member',
            condition={
                'source_translation_type': 'dynamic-ip',
                'source_translation_fallback_type': 'translated-address'}))
        params.append(VersionedParamPath(
            'source_translation_fallback_interface',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                'fallback',
                '{source_translation_fallback_type}',
                'interface')),
            condition={
                'source_translation_type': 'dynamic-ip',
                'source_translation_fallback_type': 'interface-address'}))
        params.append(VersionedParamPath(
            'source_translation_fallback_ip_type',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                'fallback',
                '{source_translation_fallback_type}',
                '{source_translation_fallback_ip_type}')),
            values=('ip', 'floating-ip'),
            default='ip',
            condition={
                'source_translation_type': 'dynamic-ip',
                'source_translation_fallback_type': 'interface-address'}))
        params.append(VersionedParamPath(
            'source_translation_fallback_ip_address',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                'fallback',
                '{source_translation_fallback_type}',
                '{source_translation_fallback_ip_type}')),
            condition={
                'source_translation_type': 'dynamic-ip',
                'source_translation_fallback_type': 'interface-address'}))
        params.append(VersionedParamPath(
            'source_translation_static_translated_address',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                'translated-address')),
            condition={
                'source_translation_type': 'static-ip'}))
        params.append(VersionedParamPath(
            'source_translation_static_bi_directional',
            vartype='yesno',
            path='/'.join((
                'source-translation',
                '{source_translation_type}',
                'bi-directional')),
            condition={
                'source_translation_type': 'static-ip'}))
        params.append(VersionedParamPath(
            'destination_translated_address',
            path='destination-translation/translated-address'))
        params.append(VersionedParamPath(
            'destination_translated_port', vartype='int',
            path='destination-translation/translated-port'))
        params.append(VersionedParamPath(
            'ha_binding', path='active-active-device-binding',
            values=('primary', 'both', '0', '1')))
        params.append(VersionedParamPath(
            'disabled', vartype='yesno', path='disabled'))
        params.append(VersionedParamPath(
            'negate_target', path='target/negate', vartype='yesno'))
        params.append(VersionedParamPath(
            'target', path='target/devices', vartype='entry'))
        params.append(VersionedParamPath(
            'tag', path='tag', vartype='member'))

        self._params = tuple(params)



import random

import pytest

from pandevice import network


def random_name():
    return ''.join(
        random.choice('abcdefghijklmnopqrstuvwxyz')
        for x in range(10)
    )


def random_ip(netmask=None):
    return '{0}.{1}.{2}.{3}{4}'.format(
        random.randint(11, 150),
        random.randint(1, 200),
        random.randint(1, 200),
        1 if netmask is not None else random.randint(2, 200),
        netmask or '',
    )


def random_ipv6(ending=None):
    if ending is None:
        return ':'.join(
            '{0:04x}'.format(random.randint(1, 65535))
            for x in range(8)
        )
    else:
        return '{0:04x}:{1:04x}:{2:04x}:{3:04x}::{4}'.format(
            random.randint(1, 65535), random.randint(1, 65535),
            random.randint(1, 65535), random.randint(1, 65535), ending)


def random_mac():
    return ':'.join(
        '{0:02x}'.format(random.randint(0, 255))
        for x in range(6)
    )


def get_available_interfaces(con, num=1):
    ifaces = network.EthernetInterface.refreshall(con, add=False)
    ifaces = set(x.name for x in ifaces)

    all_interfaces = set('ethernet1/{0}'.format(x)
                         for x in range(1, 10))
    available = all_interfaces.difference(ifaces)

    ans = []
    while len(ans) != num:
        # Raises KeyError
        ans.append(available.pop())

    return ans


class FwFlow(object):
    def test_01_setup_dependencies(self, fw, state_map):
        state = state_map.setdefault(fw)
        state.err = False
        state.fail_func = pytest.skip

        try:
            self.create_dependencies(fw, state)
        except Exception as e:
            print('SETUP ERROR: {0}'.format(e))
            state.err = True
            pytest.skip('Setup failed')

    def create_dependencies(self, fw, state):
        pass

    def sanity(self, fw, state_map):
        state = state_map.setdefault(fw)
        if state.err:
            state.fail_func('prereq failed')

        return state

    def test_02_create(self, fw, state_map):
        state = self.sanity(fw, state_map)

        state.fail_func = pytest.xfail
        state.err = True
        self.setup_state_obj(fw, state)
        state.obj.create()
        state.err = False

    def setup_state_obj(self, fw, state):
        pass

    def test_03_refreshall(self, fw, state_map):
        state = self.sanity(fw, state_map)

        objs = state.obj.refreshall(state.obj.parent, add=False)
        assert len(objs) >= 1

    def test_04_update(self, fw, state_map):
        state = self.sanity(fw, state_map)

        self.update_state_obj(fw, state)
        state.obj.apply()

    def update_state_obj(self, fw, state):
        pass

    def test_97_delete(self, fw, state_map):
        state = self.sanity(fw, state_map)

        state.obj.delete()

    def test_98_cleanup_dependencies(self, fw, state_map):
        state = state_map.setdefault(fw)
        self.cleanup_dependencies(fw, state)

    def cleanup_dependencies(self, fw, state):
        pass

    def test_99_removeall(self, fw, state_map):
        fw.removeall()


class DevFlow(object):
    def test_01_setup_dependencies(self, dev, state_map):
        state = state_map.setdefault(dev)
        state.err = False
        state.fail_func = pytest.skip

        try:
            self.create_dependencies(dev, state)
        except Exception as e:
            print('SETUP ERROR: {0}'.format(e))
            state.err = True
            pytest.skip('Setup failed')

    def create_dependencies(self, dev, state):
        pass

    def sanity(self, dev, state_map):
        state = state_map.setdefault(dev)
        if state.err:
            state.fail_func('prereq failed')

        return state

    def test_02_create(self, dev, state_map):
        state = self.sanity(dev, state_map)

        state.fail_func = pytest.xfail
        state.err = True
        self.setup_state_obj(dev, state)
        state.obj.create()
        state.err = False

    def setup_state_obj(self, dev, state):
        pass

    def test_03_refreshall(self, dev, state_map):
        state = self.sanity(dev, state_map)

        objs = state.obj.refreshall(state.obj.parent, add=False)
        assert len(objs) >= 1

    def test_04_update(self, dev, state_map):
        state = self.sanity(dev, state_map)

        self.update_state_obj(dev, state)
        state.obj.apply()

    def update_state_obj(self, dev, state):
        pass

    def test_97_delete(self, dev, state_map):
        state = self.sanity(dev, state_map)

        state.obj.delete()

    def test_98_cleanup_dependencies(self, dev, state_map):
        state = state_map.setdefault(dev)

        self.cleanup_dependencies(dev, state)

    def cleanup_dependencies(self, dev, state):
        pass

    def test_99_removeall(self, dev, state_map):
        dev.removeall()


class PanoFlow(object):
    def test_01_setup_dependencies(self, pano, state_map):
        state = state_map.setdefault(pano)
        state.err = False
        state.fail_func = pytest.skip

        try:
            self.create_dependencies(pano, state)
        except Exception as e:
            print('SETUP ERROR: {0}'.format(e))
            state.err = True
            pytest.skip('Setup failed')

    def create_dependencies(self, pano, state):
        pass

    def sanity(self, pano, state_map):
        state = state_map.setdefault(pano)
        if state.err:
            state.fail_func('prereq failed')

        return state

    def test_02_create(self, pano, state_map):
        state = self.sanity(pano, state_map)

        state.fail_func = pytest.xfail
        state.err = True
        self.setup_state_obj(pano, state)
        state.obj.create()
        state.err = False

    def setup_state_obj(self, pano, state):
        pass

    def test_03_refreshall(self, pano, state_map):
        state = self.sanity(pano, state_map)

        objs = state.obj.refreshall(state.obj.parent, add=False)
        assert len(objs) >= 1

    def test_04_update(self, pano, state_map):
        state = self.sanity(pano, state_map)

        self.update_state_obj(pano, state)
        state.obj.apply()

    def update_state_obj(self, pano, state):
        pass

    def test_97_delete(self, pano, state_map):
        state = self.sanity(pano, state_map)

        state.obj.delete()

    def test_98_cleanup_dependencies(self, pano, state_map):
        state = state_map.setdefault(pano)

        self.cleanup_dependencies(pano, state)

    def cleanup_dependencies(self, pano, state):
        pass

    def test_99_removeall(self, pano, state_map):
        pano.removeall()



# Copyright (c) 2014, Palo Alto Networks
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import unittest

import pandevice


class TestPanOSVersion(unittest.TestCase):
    """
    [Test section: PanOSVersion]

    Verify that PanOSVersion comparisons operate correctly.

    """
    def setUp(self):

        self.c1 = pandevice.PanOSVersion('7.0.0-c1')
        self.c2 = pandevice.PanOSVersion('7.0.0-c2')
        self.b1 = pandevice.PanOSVersion('7.0.0-b1')
        self.b2 = pandevice.PanOSVersion('7.0.0-b2')
        self.m1 = pandevice.PanOSVersion('7.0.0')
        self.h1 = pandevice.PanOSVersion('7.0.0-h1')
        self.h2 = pandevice.PanOSVersion('7.0.0-h2')
        self.m2 = pandevice.PanOSVersion('7.0.1')
        self.h3 = pandevice.PanOSVersion('7.0.1-h1')

    def test_version_parsed(self):
        expected = [7, 0, 0, '-', 'c', 1]

        ret_val = self.c1.version

        self.assertEqual(expected, ret_val)

    def test_str_returns_string_version(self):
        expected = '7.0.0-c1'

        ret_val = str(self.c1)

        self.assertEqual(expected, ret_val)

    def test_candidate_version_is_less_than_next_candidate_version(self):
        self.assertTrue(self.c1 < self.c2)

    def test_beta_version_is_less_than_next_beta_version(self):
        self.assertTrue(self.b1 < self.b2)

    def test_release_version_is_less_than_next_release_version(self):
        self.assertTrue(self.m1 < self.m2)

    def test_hotfix_version_is_less_than_next_hotfix_version_with_same_version_number(self):
        self.assertTrue(self.h1 < self.h2)

    def test_candidate_version_is_less_than_same_beta_version(self):
        self.assertTrue(self.c1 < self.b1)

    def test_candidate_version_is_less_than_same_release_version(self):
        self.assertTrue(self.c1 < self.m1)

    def test_candidate_version_is_less_than_same_hotfix_version(self):
        self.assertTrue(self.c1 < self.h1)

    def test_candidate_version_is_less_than_next_hotfix_version(self):
        self.assertTrue(self.c1 < self.h3)

    def test_beta_version_is_greater_than_same_candidate_version(self):
        self.assertTrue(self.b1 > self.c1)

    def test_beta_version_is_less_than_same_release_version(self):
        self.assertTrue(self.b1 < self.m1)

    def test_beta_version_is_less_than_same_hotfix_version(self):
        self.assertTrue(self.b1 < self.h1)

    def test_beta_version_is_less_than_next_hotfix_version(self):
        self.assertTrue(self.b1 < self.h3)

    def test_release_version_is_greater_than_same_candidate_version(self):
        self.assertTrue(self.m1 > self.c1)

    def test_release_version_is_greater_than_same_beta_version(self):
        self.assertTrue(self.m1 > self.b1)

    def test_release_version_is_less_than_same_hotfix_version(self):
        self.assertTrue(self.m1 < self.h1)

    def test_release_version_is_less_than_next_hotfix_version(self):
        self.assertTrue(self.m1 < self.h3)

    def test_hotfix_version_is_greater_than_same_candidate_version(self):
        self.assertTrue(self.h1 > self.c1)

    def test_hotfix_version_is_greater_than_same_beta_version(self):
        self.assertTrue(self.h1 > self.b1)

    def test_hotfix_version_is_greater_than_same_release_version(self):
        self.assertTrue(self.h1 > self.m1)

    def test_hotfix_version_is_less_than_next_release_version(self):
        self.assertTrue(self.h1 < self.m2)

    def test_hotfix_version_is_less_than_next_hotfix_version(self):
        self.assertTrue(self.h1 < self.h3)

    def test_release_version_is_greater_than_previous_candidate_version(self):
        self.assertTrue(self.m2 > self.c1)

    def test_release_version_is_greater_than_previous_beta_version(self):
        self.assertTrue(self.m2 > self.b1)

    def test_release_version_is_greater_than_previous_hotfix_version(self):
        self.assertTrue(self.m2 > self.h1)

    def test_release_version_is_less_than_hotfix_version(self):
        self.assertTrue(self.m2 < self.h3)

    def test_hotfix_version_is_greater_than_previous_candidate_version(self):
        self.assertTrue(self.h3 > self.c1)

    def test_hotfix_version_is_greater_than_previous_beta_version(self):
        self.assertTrue(self.h3 > self.b1)

    def test_hotfix_version_is_greater_than_previous_release_version(self):
        self.assertTrue(self.h3 > self.m1)

    def test_hotfix_version_is_greater_than_previous_hotfix_version(self):
        self.assertTrue(self.h3 > self.h1)

    def test_hotfix_version_is_greater_than_previous_same_release_version(self):
        self.assertTrue(self.h3 > self.m2)

if __name__=='__main__':
    unittest.main()



from pandevice import objects
from tests.live import testlib


class TestAddressObject(testlib.DevFlow):
    def setup_state_obj(self, dev, state):
        state.obj = objects.AddressObject(
            testlib.random_name(),
            value=testlib.random_ip(),
            type='ip-netmask',
            description='This is a test',
        )
        dev.add(state.obj)

    def update_state_obj(self, dev, state):
        state.obj.type = 'ip-range'
        state.obj.value = '10.1.1.1-10.1.1.240'

class TestStaticAddressGroup(testlib.DevFlow):
    def create_dependencies(self, dev, state):
        state.aos = [
            objects.AddressObject(testlib.random_name(), testlib.random_ip())
            for x in range(4)
        ]
        for x in state.aos:
            dev.add(x)
            x.create()

    def setup_state_obj(self, dev, state):
        state.obj = objects.AddressGroup(
            testlib.random_name(), [x.name for x in state.aos[:2]],
        )
        dev.add(state.obj)

    def update_state_obj(self, dev, state):
        state.obj.static_value = [x.name for x in state.aos[2:]]

    def cleanup_dependencies(self, dev, state):
        for x in state.aos:
            try:
                x.delete()
            except Exception:
                pass

class TestDynamicAddressGroup(testlib.DevFlow):
    def create_dependencies(self, dev, state):
        state.tags = [
            objects.Tag(testlib.random_name(),
                        color='color{0}'.format(x),
                        comments=testlib.random_name())
            for x in range(1, 5)
        ]
        for x in state.tags:
            dev.add(x)
            x.create()

    def setup_state_obj(self, dev, state):
        state.obj = objects.AddressGroup(
            testlib.random_name(),
            dynamic_value="'{0}' or '{1}'".format(
                state.tags[0].name, state.tags[1].name),
            description='This is my description',
            tag=state.tags[2].name,
        )
        dev.add(state.obj)

    def update_state_obj(self, dev, state):
        state.obj.dynamic_value = "'{0}' and '{1}'".format(
            state.tags[2].name, state.tags[3].name,
        )
        state.obj.tag = state.tags[1].name

    def cleanup_dependencies(self, dev, state):
        for x in state.tags:
            try:
                x.delete()
            except Exception:
                pass

class TestTag(testlib.DevFlow):
    def setup_state_obj(self, dev, state):
        state.obj = objects.Tag(
            testlib.random_name(),
            color='color1',
            comments='My new tag',
        )
        dev.add(state.obj)

    def update_state_obj(self, dev, state):
        state.obj.color = 'color5'
        state.obj.comments = testlib.random_name()

class TestServiceObject(testlib.DevFlow):
    def setup_state_obj(self, dev, state):
        state.obj = objects.ServiceObject(
            testlib.random_name(),
            protocol='tcp',
            source_port='1025-65535',
            destination_port='80,443,8080',
            description='My service object',
        )
        dev.add(state.obj)

    def update_state_obj(self, dev, state):
        state.obj.protocol = 'udp'
        state.obj.source_port = '12345'

class TestServiceGroup(testlib.DevFlow):
    def create_dependencies(self, dev, state):
        state.tag = None
        state.services = [
            objects.ServiceObject(
                testlib.random_name(),
                'tcp' if x % 2 == 0 else 'udp',
                destination_port=2000 + x,
                description='Service {0}'.format(x))
            for x in range(4)
        ]
        for x in state.services:
            dev.add(x)
            x.create()
        state.tag = objects.Tag(testlib.random_name(), 'color5')
        dev.add(state.tag)
        state.tag.create()

    def setup_state_obj(self, dev, state):
        state.obj = objects.ServiceGroup(
            testlib.random_name(),
            [x.name for x in state.services[:2]],
            tag=state.tag.name,
        )
        dev.add(state.obj)

    def update_state_obj(self, dev, state):
        state.obj.value = [x.name for x in state.services[2:]]

    def cleanup_dependencies(self, dev, state):
        for x in state.services:
            try:
                x.delete()
            except Exception:
                pass

        if state.tag is not None:
            try:
                state.tag.delete()
            except Exception:
                pass

# ApplicationObject
# ApplicationGroup
# ApplicationFilter
# ApplicationContainer



import random

from tests.live import testlib
from pandevice import network


class TestZoneBasic(testlib.FwFlow):
    def setup_state_obj(self, fw, state):
        state.obj = network.Zone(
            testlib.random_name(),
            mode='layer3',
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.mode = 'layer2'


class TestZone(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth_objs = []
        state.eths = testlib.get_available_interfaces(fw, 2)

        state.eth_objs.append(network.EthernetInterface(state.eths[0], 'layer2'))
        state.eth_objs.append(network.EthernetInterface(state.eths[1], 'layer3'))
        for x in state.eth_objs:
            fw.add(x)
        fw.create_type(network.EthernetInterface)

    def setup_state_obj(self, fw, state):
        state.obj = network.Zone(
            testlib.random_name(), 'layer2', state.eths[0],
            enable_user_identification=False,
            include_acl=testlib.random_ip('/24'),
            exclude_acl=testlib.random_ip('/24'),
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.mode = 'layer3'
        state.obj.interface = state.eths[1]
        state.obj.include_acl = [testlib.random_ip('/24') for x in range(2)]
        state.obj.exclude_acl = [testlib.random_ip('/24') for x in range(2)]

    def cleanup_dependencies(self, fw, state):
        try:
            fw.delete_type(network.EthernetInterface)
        except Exception:
            pass

class TestStaticMac(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.parent = None
        state.eth_objs = []

        state.eths = testlib.get_available_interfaces(fw, 2)

        for eth in state.eths:
            state.eth_objs.append(network.EthernetInterface(
                eth, 'layer2'))
            fw.add(state.eth_objs[-1])
        state.eth_objs[0].create_similar()

        state.parent = network.Vlan(
            testlib.random_name(), state.eths)
        fw.add(state.parent)
        state.parent.create()

    def setup_state_obj(self, fw, state):
        state.obj = network.StaticMac(
            testlib.random_mac(),
            state.eths[0],
        )
        state.parent.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.interface = state.eths[1]

    def cleanup_dependencies(self, fw, state):
        try:
            state.parent.delete()
        except Exception:
            pass

        try:
            state.eth_objs[0].delete_similar()
        except Exception:
            pass


class TestVlan(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth_objs = []

        state.eths = testlib.get_available_interfaces(fw, 2)

        for eth in state.eths:
            state.eth_objs.append(network.EthernetInterface(
                eth, 'layer2'))
            fw.add(state.eth_objs[-1])
        state.eth_objs[0].create_similar()

        state.vlan_interface = network.VlanInterface(
            'vlan.{0}'.format(random.randint(100, 200)))
        fw.add(state.vlan_interface)
        state.vlan_interface.create()

    def setup_state_obj(self, fw, state):
        state.obj = network.Vlan(
            testlib.random_name(), state.eths[0],
            state.vlan_interface.uid,
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.interface = state.eths[1]

    def cleanup_dependencies(self, fw, state):
        try:
            state.vlan_interface.delete()
        except Exception:
            pass

        try:
            state.eth_objs[0].delete_similar()
        except Exception:
            pass


class TestIPv6AddressOnEthernetInterface(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.parent = None
        state.eth = testlib.get_available_interfaces(fw)[0]

        state.parent = network.EthernetInterface(
            state.eth, 'layer3', testlib.random_ip('/24'))
        fw.add(state.parent)
        state.parent.create()

    def setup_state_obj(self, fw, state):
        state.obj = network.IPv6Address(
            testlib.random_ipv6(),
            False, True, False, True, 2420000, 604800, True, False)
        state.parent.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.enable_on_interface = True
        state.obj.prefix = False
        state.obj.anycast = True

    def cleanup_dependencies(self, fw, state):
        try:
            state.parent.delete()
        except Exception:
            pass


class TestIPv6AddressOnLayer3Subinterface(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth_obj = None
        state.eth = testlib.get_available_interfaces(fw)[0]

        state.eth_obj = network.EthernetInterface(
            state.eth, 'layer3', testlib.random_ip('/24'))
        fw.add(state.eth_obj)
        state.eth_obj.create()

        tag = random.randint(1, 4000)
        state.parent = network.Layer3Subinterface(
            '{0}.{1}'.format(state.eth, tag),
            tag, testlib.random_ip('/24'))
        state.eth_obj.add(state.parent)
        state.parent.create()

    def setup_state_obj(self, fw, state):
        state.obj = network.IPv6Address(
            testlib.random_ipv6(),
            False, True, False, True, 2420000, 604800, True, False)
        state.parent.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.enable_on_interface = True
        state.obj.prefix = False
        state.obj.anycast = True

    def cleanup_dependencies(self, fw, state):
        try:
            state.parent.delete()
        except Exception:
            pass

        try:
            state.eth_obj.delete()
        except Exception:
            pass


# Interface - inherited by other interface objects

class TestArpOnEthernetInterface(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth_obj = None
        state.eth = testlib.get_available_interfaces(fw)[0]

        state.eth_obj = network.EthernetInterface(
            state.eth, 'layer3', testlib.random_ip('/24'))
        fw.add(state.eth_obj)
        state.eth_obj.create()

    def setup_state_obj(self, fw, state):
        state.obj = network.Arp(
            testlib.random_ip(), '00:30:48:52:ab:cd')
        state.eth_obj.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.hw_address = '00:30:48:52:12:9a'

    def cleanup_dependencies(self, fw, state):
        try:
            state.eth_obj.delete()
        except Exception:
            pass


class TestArpOnSubinterface(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth_obj = None
        state.eth = testlib.get_available_interfaces(fw)[0]

        state.eth_obj = network.EthernetInterface(
            state.eth, 'layer3', testlib.random_ip('/24'))
        fw.add(state.eth_obj)
        state.eth_obj.create()

        tag = random.randint(1, 4000)
        state.parent = network.Layer3Subinterface(
            '{0}.{1}'.format(state.eth, tag),
            tag, testlib.random_ip('/24'))
        state.eth_obj.add(state.parent)
        state.parent.create()

    def setup_state_obj(self, fw, state):
        state.obj = network.Arp(
            testlib.random_ip(), testlib.random_mac())
        state.parent.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.hw_address = testlib.random_mac()

    def cleanup_dependencies(self, fw, state):
        try:
            state.parent.delete()
        except Exception:
            pass

        try:
            state.eth_obj.delete()
        except Exception:
            pass


class TestVirtualWire(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth_objs = []
        state.eths = testlib.get_available_interfaces(fw, 3)

        for eth in state.eths:
            state.eth_objs.append(network.EthernetInterface(
                eth, 'virtual-wire'))
            fw.add(state.eth_objs[-1])
            state.eth_objs[-1].create()

    def setup_state_obj(self, fw, state):
        state.obj = network.VirtualWire(
            testlib.random_name(),
            tag=random.randint(1, 4000),
            interface1=state.eths[0],
            interface2=state.eths[1],
            multicast=True,
            pass_through=False,
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.tag = random.randint(1, 4000)
        state.obj.interface1 = state.eths[1]
        state.obj.interface2 = state.eths[2]
        state.obj.multicast = False
        state.obj.pass_through = True

    def cleanup_dependencies(self, fw, state):
        for x in state.eth_objs:
            try:
                x.delete()
            except Exception:
                pass


# Subinterface - inherited by others
# AbstractSubinterface

class TestL3Subinterface(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.management_profile = network.ManagementProfile(
            testlib.random_name(), ping=True)
        state.eth = None

        fw.add(state.management_profile)
        state.management_profile.create()

        state.eth = testlib.get_available_interfaces(fw)[0]
        state.parent = network.EthernetInterface(
            state.eth, 'layer3', ip=testlib.random_ip('/24'),
        )
        fw.add(state.parent)
        state.parent.create()

    def setup_state_obj(self, fw, state):
        tag = random.randint(1, 4000)
        name = '{0}.{1}'.format(state.eth, tag)
        state.obj = network.Layer3Subinterface(
            name, tag, testlib.random_ip('/24'), False,
            state.management_profile, random.randint(576, 1500),
            True, None, 'This is my subeth',
            random.randint(40, 300), random.randint(60, 300),
        )
        state.parent.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.comment = 'Update the comment'
        state.obj.ip = testlib.random_ip('/24')

    def cleanup_dependencies(self, fw, state):
        try:
            state.management_profile.delete()
        except Exception:
            pass

        try:
            state.parent.delete()
        except Exception:
            pass


class TestL2Subinterface(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth = None

        state.eth = testlib.get_available_interfaces(fw)[0]
        state.parent = network.EthernetInterface(
            state.eth, 'layer2',
        )
        fw.add(state.parent)
        state.parent.create()

    def setup_state_obj(self, fw, state):
        tag = random.randint(1, 4000)
        name = '{0}.{1}'.format(state.eth, tag)
        state.obj = network.Layer2Subinterface(
            name, tag, comment='This is my L2 subinterface',
        )
        state.parent.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.comment = 'Updated comment'

    def cleanup_dependencies(self, fw, state):
        try:
            state.parent.delete()
        except Exception:
            pass


# PhysicalInterface - inherited by others

class TestL3EthernetInterface(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.management_profiles = []

        state.eth = testlib.get_available_interfaces(fw)[0]

        state.management_profiles = [
            network.ManagementProfile(testlib.random_name(),
                ping=bool(x)) for x in range(2)]
        for x in state.management_profiles:
            fw.add(x)

        state.management_profiles[0].create_similar()

    def setup_state_obj(self, fw, state):
        state.obj = network.EthernetInterface(
            state.eth, 'layer3', testlib.random_ip('/24'),
            ipv6_enabled=False,
            management_profile=state.management_profiles[0],
            mtu=random.randint(600, 1500),
            adjust_tcp_mss=True,
            link_speed='auto',
            link_duplex='auto',
            link_state='auto',
            comment='This is my interface',
            ipv4_mss_adjust=random.randint(40, 300),
            ipv6_mss_adjust=random.randint(60, 300),
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.management_profile = state.management_profiles[1]
        state.obj.mtu = random.randint(600, 1500)
        state.obj.ipv4_mss_adjust = random.randint(40, 300)
        state.obj.ipv6_mss_adjust = random.randint(60, 300)
        state.obj.comment = 'This is an update layer3 interface'

    def cleanup_dependencies(self, fw, state):
        try:
            state.management_profiles[0].delete_similar()
        except IndexError:
            pass

class TestL2EthernetInterface(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.management_profiles = []

        state.eth = testlib.get_available_interfaces(fw)[0]
        state.management_profiles = [
            network.ManagementProfile(testlib.random_name(),
                ping=bool(x)) for x in range(2)]
        for x in state.management_profiles:
            fw.add(x)

        state.management_profiles[0].create_similar()

    def setup_state_obj(self, fw, state):
        state.obj = network.EthernetInterface(
            state.eth, 'layer2',
            management_profile=state.management_profiles[0])
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.management_profile = state.management_profiles[1]

    def cleanup_dependencies(self, fw, state):
        try:
            state.management_profiles[0].delete_similar()
        except IndexError:
            pass


# AggregateInterface

class TestVlanInterface(testlib.FwFlow):
    def setup_state_obj(self, fw, state):
        state.obj = network.VlanInterface(
            'vlan.{0}'.format(random.randint(20, 5000)),
            testlib.random_ip('/24'),
            mtu=random.randint(800, 1000),
            adjust_tcp_mss=True,
            comment='Vlan interface',
            ipv4_mss_adjust=random.randint(100, 200),
            ipv6_mss_adjust=random.randint(100, 200),
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.ip = None
        state.obj.comment = 'Updated vlan'
        state.obj.enable_dhcp = True
        state.obj.create_dhcp_default_route = True
        state.obj.dhcp_default_route_metric = random.randint(50, 200)


class TestLoopbackInterface(testlib.FwFlow):
    def setup_state_obj(self, fw, state):
        state.obj = network.LoopbackInterface(
            'loopback.{0}'.format(random.randint(20, 5000)),
            testlib.random_ip(),
            mtu=random.randint(800, 1000),
            adjust_tcp_mss=True,
            comment='Some loopback interface',
            ipv4_mss_adjust=random.randint(100, 200),
            ipv6_mss_adjust=random.randint(100, 200),
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.ip = testlib.random_ip()
        state.obj.comment = 'Updated loopback'


class TestTunnelInterface(testlib.FwFlow):
    def setup_state_obj(self, fw, state):
        state.obj = network.TunnelInterface(
            'tunnel.{0}'.format(random.randint(20, 5000)),
            testlib.random_ip('/24'),
            mtu=random.randint(800, 1000),
            comment='Underground interface',
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.ip = testlib.random_ip('/24')
        state.obj.comment = 'Updated tunnel'


class TestStaticRoute(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth_obj = None
        state.eth = testlib.get_available_interfaces(fw)[0]

        state.eth_obj = network.EthernetInterface(
            state.eth, 'layer3', testlib.random_ip('/24'))
        fw.add(state.eth_obj)
        state.eth_obj.create()

        state.vr = network.VirtualRouter(
            testlib.random_name(), interface=state.eth)
        fw.add(state.vr)
        state.vr.create()

    def setup_state_obj(self, fw, state):
        state.obj = network.StaticRoute(
            testlib.random_name(),
            testlib.random_ip('/32'),
            'ip-address',
            testlib.random_ip(),
            state.eth,
            random.randint(10, 240),
            random.randint(1, 65535),
        )
        state.vr.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.destination = testlib.random_ip('/32')
        state.obj.nexthop_type = 'discard'
        state.obj.nexthop = None
        state.obj.interface = None

    def cleanup_dependencies(self, fw, state):
        try:
            state.vr.delete()
        except Exception:
            pass

        try:
            state.eth_obj.delete()
        except Exception:
            pass


class TestStaticRouteV6(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth_obj = None
        state.eth = testlib.get_available_interfaces(fw)[0]

        state.eth_obj = network.EthernetInterface(
            state.eth, 'layer3', testlib.random_ip('/24'), ipv6_enabled=True)
        fw.add(state.eth_obj)
        state.eth_obj.create()

        state.vr = network.VirtualRouter(
            testlib.random_name(), interface=state.eth)
        fw.add(state.vr)
        state.vr.create()

    def setup_state_obj(self, fw, state):
        ip = testlib.random_ipv6('')
        state.obj = network.StaticRouteV6(
            testlib.random_name(),
            destination=ip + '/64',
            nexthop_type='ipv6-address',
            nexthop=ip + '1',
            interface=state.eth,
            admin_dist=random.randint(100, 200),
            metric=random.randint(1, 65535),
        )
        state.vr.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.destination = testlib.random_ipv6('/64')
        state.obj.nexthop_type = 'discard'
        state.obj.nexthop = None
        state.obj.interface = None

    def cleanup_dependencies(self, fw, state):
        try:
            state.vr.delete()
        except Exception:
            pass

        try:
            state.eth_obj.delete()
        except Exception:
            pass


class TestVirtualRouter(testlib.FwFlow):
    def create_dependencies(self, fw, state):
        state.eth_obj = None
        state.eth = testlib.get_available_interfaces(fw)[0]

        state.eth_obj = network.EthernetInterface(
            state.eth, 'layer3', testlib.random_ip('/24'))
        fw.add(state.eth_obj)
        state.eth_obj.create()

    def setup_state_obj(self, fw, state):
        state.obj = network.VirtualRouter(
            testlib.random_name(),
            interface=state.eth,
            ad_static=random.randint(10, 240),
            ad_static_ipv6=random.randint(10, 240),
            ad_ospf_int=random.randint(10, 240),
            ad_ospf_ext=random.randint(10, 240),
            ad_ospfv3_int=random.randint(10, 240),
            ad_ospfv3_ext=random.randint(10, 240),
            ad_ibgp=random.randint(10, 240),
            ad_ebgp=random.randint(10, 240),
            ad_rip=random.randint(10, 240),
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.ad_static = random.randint(10, 240)
        state.obj.ad_rip = random.randint(10, 240)

    def cleanup_dependencies(self, fw, state):
        try:
            state.eth_obj.delete()
        except Exception:
            pass


class MakeVirtualRouter(testlib.FwFlow):
    WITH_OSPF = False
    WITH_AREA = False
    WITH_AUTH_PROFILE = False
    WITH_AREA_INTERFACE = False

    def create_dependencies(self, fw, state):
        state.eths = testlib.get_available_interfaces(fw, 2)

        state.eth_obj_v4 = network.EthernetInterface(
            state.eths[0], 'layer3', testlib.random_ip('/24'))
        fw.add(state.eth_obj_v4)

        state.eth_obj_v6 = network.EthernetInterface(
            state.eths[1], 'layer3', ipv6_enabled=True)
        fw.add(state.eth_obj_v6)

        state.eth_obj_v4.create_similar()

        state.vr = network.VirtualRouter(testlib.random_name(), state.eths)
        fw.add(state.vr)
        state.vr.create()

        if any((self.WITH_OSPF, self.WITH_AUTH_PROFILE,
               self.WITH_AREA, self.WITH_AREA_INTERFACE)):
            state.ospf = network.Ospf(
                True, testlib.random_ip())
            state.vr.add(state.ospf)

            if self.WITH_AUTH_PROFILE:
                state.auth = network.OspfAuthProfile(
                    testlib.random_name(), 'md5')
                state.ospf.add(state.auth)

            if self.WITH_AREA or self.WITH_AREA_INTERFACE:
                state.area = network.OspfArea(testlib.random_ip())
                state.ospf.add(state.area)

                if self.WITH_AREA_INTERFACE:
                    state.iface = network.OspfAreaInterface(
                        state.eths[0], True, True, 'p2mp')
                    state.area.add(state.iface)

            state.ospf.create()

    def cleanup_dependencies(self, fw, state):
        try:
            state.vr.delete()
        except Exception:
            pass

        try:
            state.eth_obj_v4.delete_similar()
        except Exception:
            pass


class TestRedistributionProfile(MakeVirtualRouter):
    def setup_state_obj(self, fw, state):
        some_ip = testlib.random_ip()

        state.obj = network.RedistributionProfile(
            testlib.random_name(),
            priority=random.randint(1, 255),
            action='no-redist',
            filter_type=['ospf', 'static', 'connect'],
            filter_interface=random.choice(state.eths),
            filter_destination=testlib.random_ip(),
            filter_nexthop=testlib.random_ip(),
            ospf_filter_pathtype=('intra-area', 'ext-1'),
            ospf_filter_area=some_ip,
            ospf_filter_tag=some_ip,
        )
        state.vr.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.action = 'redist'
        state.obj.filter_type = ('ospf', 'rip', 'bgp')
        state.obj.ospf_filter_pathtype = ('inter-area', 'ext-2')
        state.obj.bgp_filter_community = ('local-as', 'no-export')


class TestOspf(MakeVirtualRouter):
    def setup_state_obj(self, fw, state):
        state.obj = network.Ospf(
            True, testlib.random_ip(), True, True, True,
            2, 3, False, 300, False, False, 400)
        state.vr.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.enable = False
        state.obj.reject_default_route = False
        state.allow_redist_default_route = False
        state.obj.rfc1583 = False
        state.obj.spf_calculation_delay = 3
        state.obj.lsa_interval = 4
        state.obj.graceful_restart_enable = True
        state.obj.gr_helper_enable = True
        state.obj.gr_strict_lsa_checking = True


class TestOspfArea(MakeVirtualRouter):
    WITH_OSPF = True

    def setup_state_obj(self, fw, state):
        state.obj = network.OspfArea(
            testlib.random_ip(), 'normal')
        state.ospf.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.type = 'stub'
        state.obj.accept_summary = True
        state.obj.default_route_advertise = 'disable'

    def test_05_stub_area_with_default_route_advertise(self, fw, state_map):
        state = self.sanity(fw, state_map)

        state.obj.default_route_advertise = 'advertise'
        state.obj.default_route_advertise_metric = 45

        state.obj.apply()

    def test_06_nssa_area_type_ext1(self, fw, state_map):
        state = self.sanity(fw, state_map)

        state.obj.type = 'nssa'
        state.obj.default_route_advertise_type = 'ext-1'

        state.obj.apply()

    def test_07_nssa_area_type_ext2(self, fw, state_map):
        state = self.sanity(fw, state_map)

        state.obj.default_route_advertise_type = 'ext-2'

        state.obj.apply()


class TestOspfRange(MakeVirtualRouter):
    WITH_AREA = True

    def setup_state_obj(self, fw, state):
        state.obj = network.OspfRange(testlib.random_ip(), 'advertise')
        state.area.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.mode = 'suppress'


class TestOspfNssaExternalRange(MakeVirtualRouter):
    WITH_AREA = True

    def create_dependencies(self, fw, state):
        super(TestOspfNssaExternalRange, self).create_dependencies(fw, state)
        state.area.type = 'nssa'
        state.area.apply()

    def setup_state_obj(self, fw, state):
        state.obj = network.OspfNssaExternalRange(
            testlib.random_ip('/24'), 'advertise')
        state.area.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.mode = 'suppress'


class TestOspfAreaInterface(MakeVirtualRouter):
    WITH_AREA = True
    WITH_AUTH_PROFILE = True

    def setup_state_obj(self, fw, state):
        state.obj = network.OspfAreaInterface(
            random.choice(state.eths), True, True, 'broadcast', 4096, 50,
            12, 3, 4, 5, 6, state.auth.uid)
        state.area.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.enable = False
        state.obj.passive = False
        state.obj.link_type = 'p2p'

    def test_05_link_type_p2mp(self, fw, state_map):
        state = self.sanity(fw, state_map)

        state.obj.enable = True
        state.obj.link_type = 'p2mp'

        state.obj.apply()


class TestOspfNeighbor(MakeVirtualRouter):
    WITH_AREA_INTERFACE = True

    def setup_state_obj(self, fw, state):
        state.obj = network.OspfNeighbor(testlib.random_ip(), 10)
        state.iface.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.metric = 11


class TestOspfAuthProfile(MakeVirtualRouter):
    WITH_OSPF = True

    def setup_state_obj(self, fw, state):
        state.obj = network.OspfAuthProfile(
            testlib.random_name(), 'password', 'secret')
        state.ospf.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.password = 'secret2'


class TestOspfAuthProfileMd5(MakeVirtualRouter):
    WITH_AUTH_PROFILE = True

    def setup_state_obj(self, fw, state):
        state.obj = network.OspfAuthProfileMd5(
            '1', 'secret1', False)
        state.auth.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.preferred = True

    def test_05_add_second_profile_not_preferred(self, fw, state_map):
        state = self.sanity(fw, state_map)

        o = network.OspfAuthProfileMd5('2', 'secret2', False)

        state.auth.add(o)
        o.create()


class TestOspfExportRules(MakeVirtualRouter):
    WITH_OSPF = True

    def setup_state_obj(self, fw, state):
        state.obj = network.OspfExportRules(
            testlib.random_ip('/24'),
            'ext-2', testlib.random_ip(), 2048)
        state.ospf.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.new_path_type = 'ext-1'
        state.obj.metric = 5309


class TestManagementProfile(testlib.FwFlow):
    def setup_state_obj(self, fw, state):
        state.obj = network.ManagementProfile(
            testlib.random_name(),
            ping=True,
            telnet=False,
            ssh=True,
            http=False,
            http_ocsp=True,
            https=False,
            snmp=True,
            response_pages=False,
            userid_service=True,
            userid_syslog_listener_ssl=False,
            userid_syslog_listener_udp=True,
            permitted_ip=['1.2.3.4', '5.6.7.8'],
        )
        fw.add(state.obj)

    def update_state_obj(self, fw, state):
        state.obj.permitted_ip = ['9.8.7.6', ]
        state.obj.https = True
        state.obj.http_ocsp = False



try:
    from unittest import mock
except ImportError:
    import mock
import unittest
import xml.etree.ElementTree as ET

import pandevice.base
import pandevice.network
import pandevice.objects
import pandevice.policies


class TestObject(unittest.TestCase):
    OLD_CLS = None
    NEW_CLS = None
    PARAMS = ()

    def setUp(self):
        if self.OLD_CLS is None:
            raise unittest.SkipTest('OLD_CLS not defined')
        elif self.NEW_CLS is None:
            raise unittest.SkipTest('NEW_CLS not defined')

    def test_empty_objects_are_equal(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'element_str'):
            raise unittest.SkipTest('OLD_CLS does not have element_str()')

        self.assertEqual(
            old.element_str(),
            new.element_str(),
        )

    def test_positionally_populated_objects_are_equal(self):
        args = tuple(y for x, y in self.PARAMS)
        new = self.NEW_CLS('jack', *args)
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))
        old = self.OLD_CLS('jack', *args)

        if not hasattr(old, 'element_str'):
            raise unittest.SkipTest('OLD_CLS does not have element_str()')

        self.assertEqual(
            old.element_str(),
            new.element_str(),
        )

    def test_keyword_populated_objects_are_equal(self):
        kwargs = dict(self.PARAMS)
        old = self.OLD_CLS('burton', **kwargs)
        new = self.NEW_CLS('burton', **kwargs)
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'element_str'):
            raise unittest.SkipTest('OLD_CLS does not have element_str()')

        self.assertEqual(
            old.element_str(),
            new.element_str(),
        )

    def test_parsing_old_elmstring_works(self):
        kwargs = dict(self.PARAMS)
        old = self.OLD_CLS('myuid', **kwargs)
        orig = self.OLD_CLS('blah')
        new = self.NEW_CLS()
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'element'):
            raise unittest.SkipTest('OLD_CLS does not have element()')

        # We make a fresh OLD_CLS from an instantiated OLD_CLS first because
        # if there are conditional variables, they should not show up
        # in NEW_CLS, but would be in OLD_CLS.
        orig.refresh(xml=old.element())
        new.parse_xml(orig.element())

        if orig.SUFFIX == pandevice.base.ENTRY:
            self.assertEqual(orig.uid, new.uid)

        if orig.element_str() != new.element_str():
            for key, value in self.PARAMS:
                self.assertEqual(
                    getattr(orig, key),
                    getattr(new, key),
                    'Key({0}) orig({1}) vs new({2})'.format(
                        key,
                        getattr(orig, key),
                        getattr(new, key)),
                )

    def test_root_is_same(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'ROOT'):
            raise unittest.SkipTest('OLD_CLS does not have ROOT')

        self.assertEqual(old.ROOT, new.ROOT)

    def test_class_xpath_is_same(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'XPATH'):
            raise unittest.SkipTest('OLD_CLS does not have XPATH')

        self.assertEqual(old.XPATH, new.XPATH)

    def test_suffix_is_same(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'SUFFIX'):
            raise unittest.SkipTest('OLD_CLS does not have SUFFIX')

        self.assertEqual(old.SUFFIX, new.SUFFIX)

    def test_childtypes_is_same(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'CHILDTYPES'):
            raise unittest.SkipTest('OLD_CLS does not have CHILDTYPES')

        for x in old.CHILDTYPES:
            self.assertTrue(x in new.CHILDTYPES,
                            'CHILDTYPE {0} missing'.format(x))

    def test_childmethods_is_same(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'CHILDMETHODS'):
            raise unittest.SkipTest('OLD_CLS does not have CHILDMETHODS')

        for x in old.CHILDMETHODS:
            self.assertTrue(x in new.CHILDMETHODS)

    def test_ha_sync_is_same(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'HA_SYNC'):
            raise unittest.SkipTest('OLD_CLS does not have HA_SYNC')

        self.assertEqual(old.HA_SYNC, new.HA_SYNC)

    def test_name_is_same(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'NAME'):
            raise unittest.SkipTest('OLD_CLS does not have NAME')

        self.assertEqual(old.NAME, new.NAME)

    def test_xpath_function_is_same(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'xpath'):
            raise unittest.SkipTest('OLD_CLS does not have xpath()')

        self.assertEqual(old.xpath(), new.xpath())

    def test_xpath_short_is_same(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        if not hasattr(old, 'xpath_short'):
            raise unittest.SkipTest('OLD_CLS does not have xpath_short()')

        self.assertEqual(old.xpath_short(), new.xpath_short())

    def test_xpath_import_is_same(self):
        if not hasattr(self.OLD_CLS, 'XPATH_IMPORT'):
            raise unittest.SkipTest('No XPATH_IMPORT for this class')

        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        self.assertEqual(old.XPATH_IMPORT, new.XPATH_IMPORT)

    def test_has_the_same_functions(self):
        old = self.OLD_CLS('foo')
        new = self.NEW_CLS('foo')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        ignore_functions = (
            'variables',
            '_variables',
            'vars_with_mode',
        )
        funcs = [x for x in dir(old) if callable(getattr(old, x))
                 and x not in ignore_functions]
        for x in funcs:
            self.assertTrue(hasattr(new, x) and callable(getattr(new, x)),
                            'NEW_CLS missing function {0}'.format(x))

    def test_access_to_set_vlan_function(self):
        old = self.OLD_CLS('jack')
        new = self.NEW_CLS('jack')
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        new._set_reference = mock.Mock(return_value='foo')

        if hasattr(old, 'set_vlan'):
            ret_val = new.set_vlan('burton')

            self.assertEqual('foo', ret_val)
            self.assertEqual(1, new._set_reference.call_count)
        else:
            with self.assertRaises(AttributeError):
                new.set_vlan('burton')

    def test_empty_name_is_equal(self):
        try:
            old = self.OLD_CLS()
        except TypeError:
            raise unittest.SkipTest('Cannot create empty class of this type')

        new = self.NEW_CLS()
        new.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        self.assertEqual(old.uid, new.uid)


class TestVersionedObject(unittest.TestCase):
    MASKED_VALUE = 'old default'
    DEFAULT_VALUE = 'new default'
    MIDDLE_VALUE = '6.1 profile'
    NEWEST_VALUE = '7.0 profile'

    def setUp(self):
        self.obj = pandevice.base.VersioningSupport()
        self.obj.add_profile('1.0.0', self.MASKED_VALUE)
        self.obj.add_profile('1.0.0', self.DEFAULT_VALUE)
        self.obj.add_profile('6.1.0', self.MIDDLE_VALUE)
        self.obj.add_profile('7.0.0', self.NEWEST_VALUE)

    def _check(self, version, expected):
        result = self.obj._get_versioned_value(version)

        self.assertEqual(expected, result)

    def test__get_versioned_value_on_higher_panos_version_gets_newest_value(self):
        self._check((8, 0, 0), self.NEWEST_VALUE)

    def test__get_versioned_value_on_lower_panos_version_gets_oldest_value(self):
        self._check((0, 0, 0), self.MASKED_VALUE)

    def test__get_versioned_value_for_exact_panos_version_match(self):
        self._check((6, 1, 0), self.MIDDLE_VALUE)

    def test__get_versioned_value_for_inbetween_panos_version(self):
        self._check((6, 5, 0), self.MIDDLE_VALUE)

    def test__get_versioned_value_gets_newer_value_with_multiple_exact_version_profiles(self):
        self._check((1, 0, 0), self.DEFAULT_VALUE)

    def test_add_profile_raises_error_on_adding_lower_version_after_adding_a_higher_version(self):
        self.assertRaises(
            ValueError,
            self.obj.add_profile, '5.5.5', 'foo')

if __name__=='__main__':
    unittest.main()



import datetime
import random

from tests.live import testlib


class TestUserId(object):
    TAGS_PER = 10

    def test_00(self, fw, state_map):
        fw.userid.clear_registered_ip()

    def test_01(self, fw, state_map):
        state = state_map.setdefault(fw)
        state.data = [
            (testlib.random_ip(), [testlib.random_name()
                                   for x in range(self.TAGS_PER)])
            for x in range(10)]

        for ip, tags in state.data:
            fw.userid.register(ip, tags)

        ans = fw.userid.get_registered_ip()
        assert len(ans) == len(state.data)

        for key, tags in ans.items():
            for state_key, state_tags in state.data:
                if state_key != key:
                    continue
                assert set(state_tags) == set(tags)

    def test_02(self, fw, state_map):
        state = state_map.setdefault(fw)

        for ip, tags in state.data:
            fw.userid.unregister(ip, tags)

        assert len(fw.userid.get_registered_ip()) == 0

    def test_03_looper(self, fw, state_map):
        ips = [testlib.random_ip() for x in range(10)]
        tags = [testlib.random_name() for x in range(10)]
        tag_set = set(tags)
        one_ip = random.choice(ips)

        print('ips: {0}'.format(ips))
        print('tags: {0}'.format(tags))
        assert len(fw.userid.get_registered_ip()) == 0

        for num in range(1, 6):
            print('Loop {0}...'.format(num))

            # Register stuff
            fw.userid.register(ips, tags)
            ans = fw.userid.get_registered_ip()
            for ip in ips:
                assert ip in ans
                assert set(ans[ip]) == set(tag_set)

            # Unregister one at random
            first_selection = random.choice(tags)
            print('first_selection = {0}'.format(first_selection))
            fw.userid.unregister(ips, first_selection)
            ans = fw.userid.get_registered_ip()
            assert len(ans[one_ip]) == 9

            # Unregister one more at random
            second_selection = first_selection
            while second_selection == first_selection:
                second_selection = random.choice(tags)
            print('second_selection = {0}'.format(second_selection))
            fw.userid.unregister(ips, second_selection)
            ans = fw.userid.get_registered_ip()
            assert len(ans[one_ip]) == 8

            # Unregister everything, which includes already deleted things
            fw.userid.unregister(ips, tags)
            ans = fw.userid.get_registered_ip()
            assert ans == {}



import os
import random

import pytest

from pandevice import firewall
from pandevice import panorama


live_devices = {}
one_fw_per_version = []
one_device_type_per_version = []
one_panorama_per_version = []
ha_pairs = []
panorama_fw_combinations = []


def desc(pano=None, fw=None):
    ans = []
    if pano is not None:
        ans.append('{0}.{1}Pano'.format(*pano))
        if fw is not None:
            ans.append('With')
    if fw is not None:
        ans.append('{0}.{1}NGFW'.format(*fw))
    return ''.join(ans)


def init():
    '''
    Environment variables:
        USERNAME
        PASSWORD
        PANORAMAS
        FIREWALLS
    '''
    global live_devices
    global one_fw_per_version
    global one_device_per_version
    global one_panorama_per_version
    global ha_pairs
    global panorama_fw_combinations

    # Get os.environ stuff to set the live_devices global.
    try:
        username = os.environ['USERNAME']
        password = os.environ['PASSWORD']
        panos = os.environ['PANORAMAS'].split()
        fws = os.environ['FIREWALLS'].split()
    except KeyError as e:
        print('NOT RUNNING LIVE TESTS - missing "{0}"'.format(e))
        return

    # Add each panorama to the live_devices.
    for hostname in panos:
        c = panorama.Panorama(hostname, username, password)
        try:
            c.refresh_system_info()
        except Exception as e:
            raise ValueError('Failed to connect to panorama {0}: {1}'.format(
                             hostname, e))

        # There should only be one panorama per version.
        version = c._version_info
        if version in live_devices:
            raise ValueError('Two panoramas, same version: {0} and {1}'.format(
                             live_devices[version]['pano'].hostname, hostname))
        live_devices.setdefault(version, {'fws': [], 'pano': None})
        live_devices[version]['pano'] = c

    # Add each firewall to the live_devices.
    for hostname in fws:
        c = firewall.Firewall(hostname, username, password)
        try:
            c.refresh_system_info()
        except Exception as e:
            raise ValueError('Failed to connect to firewall {0}: {1}'.format(
                             hostname, e))

        # Multiple firewalls are allowed per version, but only ever the first
        # two will be used.
        version = c._version_info
        live_devices.setdefault(version, {'fws': [], 'pano': None})
        live_devices[version]['fws'].append(c)

    # Set:
    #   one_fw_per_version
    #   one_device_type_per_version
    #   one_panorama_per_version
    for version in live_devices:
        pano = live_devices[version]['pano']
        fws = live_devices[version]['fws']
        if fws:
            fw = random.choice(fws)
            one_device_type_per_version.append((fw, desc(fw=version)))
            one_fw_per_version.append((fw, desc(fw=version)))
        if pano is not None:
            one_panorama_per_version.append((pano, desc(pano=version)))
            one_device_type_per_version.append((pano, desc(pano=version)))

    # Set: ha_pairs
    for version in live_devices:
        fws = live_devices[version]['fws']
        if len(fws) >= 2:
            ha_pairs.append((fws[:2], version))

    # Set panorama_fw_combinations
    for pano_version in live_devices:
        pano = live_devices[pano_version]['pano']
        if pano is None:
            continue

        for fw_version in live_devices:
            fws = live_devices[fw_version]['fws']
            if not fws or pano_version < fw_version:
                continue

            fw = random.choice(fws)
            panorama_fw_combinations.append((
                (pano, fw), desc(pano_version, fw_version),
            ))


# Invoke the init() to set globals for our tests.
init()


def pytest_report_header(config):
    if not one_device_type_per_version:
        ans = [
            'Skipping live tests; no devices in the config',
        ]
    else:
        ans = ['Given the following devices:', ]
        for v in sorted(live_devices.keys()):
            line = ['* Version:{0}.{1}.{2}'.format(*v), ]
            if live_devices[v]['pano'] is not None:
                line.append('Panorama:{0}'.format(live_devices[v]['pano'].hostname))
            for fw in live_devices[v]['fws']:
                line.append('NGFW:{0}'.format(fw.hostname))
            ans.append(' '.join(line))

    return ans


# Order tests alphabetically.  This is needed because by default pytest gets
# the tests of the current class, executes them, then walks the inheritance
# to get parent tests, which is not what we want.
def pytest_collection_modifyitems(items):
    grouping = {}
    lookup = {}
    reordered = []

    for x in items:
        location, tc = x.nodeid.rsplit('::', 1)
        lookup[(location, tc)] = x
        grouping.setdefault(location, [])
        grouping[location].append(tc)

    for location in sorted(grouping.keys()):
        tests = sorted(grouping[location])
        for tc in tests:
            reordered.append(lookup[(location, tc)])

    items[:] = reordered


# Define a state fixture.
class State(object):
    pass


class StateMap(object):
    def __init__(self):
        self.config = {}

    def setdefault(self, *x):
        key = tuple(d.hostname for d in x)
        return self.config.setdefault(key, State())


@pytest.fixture(scope='class')
def state_map(request):
    yield StateMap()


# Define parametrized fixtures.
@pytest.fixture(
    scope='session',
    params=[x[0] for x in one_fw_per_version],
    ids=[x[1] for x in one_fw_per_version],
)
def fw(request):
    return request.param


@pytest.fixture(
    scope='session',
    params=[x[0] for x in one_device_type_per_version],
    ids=[x[1] for x in one_device_type_per_version],
)
def dev(request):
    return request.param


@pytest.fixture(
    scope='session',
    params=[x[0] for x in one_panorama_per_version],
    ids=[x[1] for x in one_panorama_per_version],
)
def pano(request):
    return request.param


@pytest.fixture(
    scope='session',
    params=[x[0] for x in panorama_fw_combinations],
    ids=[x[1] for x in panorama_fw_combinations],
)
def pairing(request):
    return request.param



#!/usr/bin/env python

# Copyright (c) 2014, Palo Alto Networks
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""Device module contains objects that exist in the 'Device' tab in the firewall GUI"""

from pandevice.base import PanObject, Root, MEMBER, ENTRY
from pandevice.base import VarPath as Var
from pandevice.base import VersionedPanObject
from pandevice.base import VersionedParamPath

# import other parts of this pandevice package
from pandevice import getlogger
import pandevice.errors as err

logger = getlogger(__name__)


class VsysResources(VersionedPanObject):
    """Resource constraints for a Vsys

    Args:
        max_security_rules (int): Maximum security rules
        max_nat_rules (int): Maximum nat rules
        max_ssl_decryption_rules (int): Maximum ssl decryption rules
        max_qos_rules (int): Maximum QOS rules
        max_application_override_rules (int): Maximum application override rules
        max_pbf_rules (int): Maximum policy based forwarding rules
        max_cp_rules (int): Maximum captive portal rules
        max_dos_rules (int): Maximum DOS rules
        max_site_to_site_vpn_tunnels (int): Maximum site-to-site VPN tunnels
        max_concurrent_ssl_vpn_tunnels (int): Maximum ssl VPN tunnels
        max_sessions (int): Maximum sessions

    """
    NAME = None
    ROOT = Root.VSYS

    def _setup(self):
        # xpaths
        self._xpaths.add_profile(value='/import/resource')

        # params
        params = []

        int_params = ("max-security-rules", "max-nat-rules",
            "max-ssl-decryption-rules", "max-qos-rules",
            "max-application-override-rules", "max-pbf-rules",
            "max-cp-rules", "max-dos-rules", "max-site-to-site-vpn-tunnels",
            "max-concurrent-ssl-vpn-tunnels", "max-sessions",
        )
        for x in int_params:
            params.append(VersionedParamPath(x, path=x, vartype='int'))

        self._params = tuple(params)


class Vsys(VersionedPanObject):
    """Virtual System (VSYS)

    You can interact with virtual systems in two different ways:

    **Method 1**. Use a :class:`pandevice.firewall.Firewall` object with the 'vsys'
    variable set to a vsys identifier (eg. 'vsys2'). In this case,
    you don't need to use this Vsys class. Add other PanObject instances
    (like :class:`pandevice.objects.AddressObject`) to the Firewall instance

    **Method 2**. Add an instance of this Vsys class to a :class:`pandevice.firewall.Firewall`
    object. It is best practice to set the Firewall instance's 'shared'
    variable to True when using this method. Add other PanObject instances
    (like :class:`pandevice.objects.AddressObject`) to the Vsys instance.

    Args:
        name (str): Vsys identifier (eg. 'vsys1', 'vsys5', etc)
        display_name (str): Friendly name of the vsys
        interface (list): A list of strings with names of interfaces
            or a list of :class:`pandevice.network.Interface` objects
        vlans (list): A list of strings of VLANs
        virtual_wires (list): A list of strings of virtual wires
        virtual_routers (list): A list of strings of virtual routers
        visible_vsys (list): A list of strings of the vsys visible
        dns_proxy (str): DNS Proxy server
        decrypt_forwarding (bool): Allow forwarding of decrypted content

    """
    ROOT = Root.DEVICE
    SUFFIX = ENTRY

    def _setup(self):
        # xpaths
        self._xpaths.add_profile(value='/vsys')

        # params
        params = []

        params.append(VersionedParamPath(
            'display_name', path='display-name'))
        params.append(VersionedParamPath(
            'interface', vartype='member',
            path='import/network/interface'))
        params.append(VersionedParamPath(
            'vlans', vartype='member',
            path='import/network/vlan'))
        params.append(VersionedParamPath(
            'virtual_wires', vartype='member',
            path='import/network/virtual-wire'))
        params.append(VersionedParamPath(
            'virtual_routers', vartype='member',
            path='import/network/virtual-router'))
        params.append(VersionedParamPath(
            'visible_vsys', vartype='member',
            path='import/visible-vsys'))
        params.append(VersionedParamPath(
            'dns_proxy', path='import/dns-proxy'))
        params.append(VersionedParamPath(
            'decrypt_forwarding', vartype='yesno',
            path='setting/ssl-decrypt/allow-forward-decrypted-content'))

        self._params = tuple(params)

    def xpath_vsys(self):
        if self.name == "shared":
            return "/config/shared"
        elif self.name is None:
            return self._root_xpath_vsys('vsys1')
        else:
            return self._root_xpath_vsys(self.name)

    def _build_xpath(self, root):
        if root == Root.VSYS:
            return super(Vsys, self)._build_xpath(root)
        else:
            # If original object is not a VSYS Root, then bypass the vsys part of the xpath
            if self.parent is None:
                return ""
            else:
                return self.parent._build_xpath(root)

    @property
    def vsys(self):
        return self.name

    @vsys.setter
    def vsys(self, value):
        self.name = value


class NTPServer(PanObject):
    """A primary or secondary NTP server

    This is an abstract base class, do not instantiate it.

    Args:
        address (str): The IP address of the NTP server

    """
    # TODO: Add authentication
    # TODO: Add PAN-OS pre-7.0 support

    XPATH = "/ntp-servers/primary-ntp-server"

    def __init__(self, *args, **kwargs):
        if type(self) == NTPServer:
            raise err.PanDeviceError("Do not instantiate class. Please use a subclass.")
        super(NTPServer, self).__init__(*args, **kwargs)

    @classmethod
    def variables(cls):
        return (
            Var("ntp-server-address", "address"),
        )


class NTPServerPrimary(NTPServer):
    """A primary NTP server

    Add to a :class:`pandevice.device.SystemSettings` object

    Args:
        address (str): IP address or hostname of NTP server

    """
    XPATH = "/ntp-servers/primary-ntp-server"


class NTPServerSecondary(NTPServer):
    """A secondary NTP server

    Add to a :class:`pandevice.device.SystemSettings` object

    Args:
        address (str): IP address or hostname of NTP server

    """
    XPATH = "/ntp-servers/secondary-ntp-server"


class SystemSettings(PanObject):
    """Firewall or Panorama device system settings

    Add only one of these to a parent object.

    Args:
        hostname (str): The hostname of the device
        domain (str): The domain of the device
        ip_address (str): Management interface IP address
        netmask (str): Management interface netmask
        default_gateway (str): Management interface default gateway
        ipv6_address (str): Management interface IPv6 address
        ipv6_default_gateway (str): Management interface IPv6 default gateway
        dns_primary (str): Primary DNS server IP address
        dns_secondary (str): Secondary DNS server IP address
        timezone (str): Device timezone
        panorama (str): IP address of primary Panorama
        panorama2 (str):  IP address of secondary Panorama
        login_banner (str): Login banner text
        update_server (str): IP or hostname of the update server

    """
    NAME = None
    ROOT = Root.DEVICE
    XPATH = "/deviceconfig/system"
    HA_SYNC = False
    CHILDTYPES = (
        "device.NTPServerPrimary",
        "device.NTPServerSecondary",
    )

    @classmethod
    def variables(cls):
        return (
            Var("hostname"),
            Var("domain"),
            Var("ip-address"),
            Var("netmask"),
            Var("default-gateway"),
            Var("ipv6-address"),
            Var("ipv6-default-gateway"),
            Var("dns-setting/servers/primary", "dns_primary"),
            Var("dns-setting/servers/secondary", "dns_secondary"),
            Var("timezone"),
            Var("panorama-server", "panorama"),
            Var("panorama-server-2", "panorama2"),
            Var("login-banner"),
            Var("update-server"),
        )


class PasswordProfile(VersionedPanObject):
    """Password profile object

    Args:
        name (str): Password profile name
        expiration (int): Number of days until the password expires
        warning (int): Number of days warning before password expires
        login_count (int): Post expiration admin login count
        grace_period (int): Post expiration grace period

    """
    ROOT = Root.MGTCONFIG
    SUFFIX = ENTRY

    def _setup(self):
        # xpaths
        self._xpaths.add_profile(value='/password-profile')

        # params
        params = []

        params.append(VersionedParamPath(
            'expiration', vartype='int',
            path='password-change/expiration-period'))
        params.append(VersionedParamPath(
            'warning', vartype='int',
            path='password-change/expiration-warning-period'))
        params.append(VersionedParamPath(
            'login_count', vartype='int',
            path='password-change/post-expiration-admin-login-count'))
        params.append(VersionedParamPath(
            'grace_period', vartype='int',
            path='password-change/post-expiration-grace-period'))

        self._params = tuple(params)


class Administrator(VersionedPanObject):
    """Administrator object

    Args:
        name (str): Admin name
        authentication_profile (str): The authentication profile
        web_client_cert_only (bool): Use only client certificate authentication (Web)
        superuser (bool): Admin type - superuser
        superuser_read_only (bool): Admin type - superuser, read only
        panorama_admin (bool): Panonrama - a panorama admin only
        device_admin (bool): Admin type - device admin
        device_admin_read_only (bool): Admin type - device admin, read only
        vsys (list/str): Physical firewalls: the vsys this admin should manage
        vsys_read_only (list/str): Physical firewalls: the vsys this read only admin should manage
        ssh_public_key (str): Use Public Key Authentication (SSH)
        role_profile (str): The role based profile
        password_hash (encrypted str): The encrypted password
        password_profile (str): The password profile for this user

    """
    ROOT = Root.MGTCONFIG
    SUFFIX = ENTRY

    def _setup(self):
        # xpaths
        self._xpaths.add_profile(value='/users')

        # params
        params = []

        params.append(VersionedParamPath(
            'authentication_profile', path='authentication-profile'))
        params.append(VersionedParamPath(
            'web_client_cert_only', vartype='yesno',
            path='client-certificate-only'))
        params.append(VersionedParamPath(
            'superuser', vartype='yesno',
            path='permissions/role-based/superuser'))
        params.append(VersionedParamPath(
            'superuser_read_only', vartype='yesno',
            path='permissions/role-based/superreader'))
        params.append(VersionedParamPath(
            'panorama_admin', vartype='yesno',
            path='permissions/role-based/panorama-admin'))
        params.append(VersionedParamPath(
            'device_admin', vartype='exist',
            path='permissions/role-based/deviceadmin'))
        params.append(VersionedParamPath(
            'device_admin_read_only', vartype='exist',
            path='permissions/role-based/devicereader'))
        params.append(VersionedParamPath(
            'vsys', vartype='member',
            path='permissions/role-based/vsysadmin/entry vsys_device/vsys'))
        params.append(VersionedParamPath(
            'vsys_read_only', vartype='member',
            path='permissions/role-based/vsysreader' +
                 '/entry vsys_read_only_device/vsys'))
        params.append(VersionedParamPath(
            'ssh_public_key', path='public-key'))
        params.append(VersionedParamPath(
            'role_profile', path='permissions/role-based/custom/profile'))
        params.append(VersionedParamPath(
            'password_hash', path='phash', vartype='encrypted'))
        params.append(VersionedParamPath(
            'password_profile', path='password-profile'))
        params.append(VersionedParamPath(
            'vsys_device', exclude=True, vartype='entry',
            path='permissions/role-based/vsysadmin',
            default='localhost.localdomain'))
        params.append(VersionedParamPath(
            'vsys_read_only_device', exclude=True, vartype='entry',
            path='permissions/role-based/vsysreader',
            default='localhost.localdomain'))

        self._params = tuple(params)

    def change_password(self, new_password):
        """Update the password.

        **Modifies the live device**

        Args:
            new_password (str): The new password for this user.

        """
        dev = self.nearest_pandevice()
        self.password_hash = dev.request_password_hash(new_password)
        self.update('password_hash')



# Copyright (c) 2014, Palo Alto Networks
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
try:
    from unittest import mock
except ImportError:
    import mock
import unittest
import pan.xapi

import pandevice.base as Base
import pandevice.device
import pandevice.errors as Err
import pandevice.firewall
import pandevice.ha
import pandevice.network
import pandevice.objects
import pandevice.panorama


class TestNearestPandevice(unittest.TestCase):
    """
    [Test section: nearest_pandevice()]

    Verify that `nearest_pandevice()` returns the correct PanObject.

    All tests have a setup of the following:
        Panorama > DeviceGroup > Firewall > AddressObject

    In this configuration, running .nearest_pandevice() should return the
    panorama device in all cases but running it from the AddressObject.
    """
    def setUp(self):
        self.panorama = pandevice.panorama.Panorama(
            'foo', 'bar', 'baz', 'apikey')
        self.device_group = pandevice.panorama.DeviceGroup(
            'My pandevice Group')
        self.firewall = pandevice.firewall.Firewall(
            'foo', 'bar', 'baz', 'apikey')
        self.address_object = pandevice.objects.AddressObject(
            'webserver', '192.168.1.100', description='Intranet web server',
            tag=['http', 'https'])

        self.firewall.add(self.address_object)
        self.device_group.add(self.firewall)
        self.panorama.add(self.device_group)

        self.assertEqual(self.firewall, self.address_object.parent)
        self.assertEqual([], self.address_object.children)
        self.assertEqual(self.device_group, self.firewall.parent)
        self.assertEqual([self.address_object, ], self.firewall.children)
        self.assertEqual(self.panorama, self.device_group.parent)
        self.assertEqual([self.firewall, ], self.device_group.children)
        self.assertEqual(None, self.panorama.parent)
        self.assertEqual([self.device_group, ], self.panorama.children)

    def test_nearest_pandevice_from_addressobject_in_pano_dg_fw_ao_chain(self):
        """Runs nearest_pandevice() on the AddressObject.

        This should return the Firewall PanDevice.
        """
        ret_val = self.address_object.nearest_pandevice()

        self.assertEqual(self.firewall, ret_val)

    def test_nearest_pandevice_from_firewall_in_pano_dg_fw_ao_chain(self):
        """Runs nearest_pandevice() on the firewall.

        This should return the Panorama PanDevice.
        """
        ret_val = self.firewall.nearest_pandevice()

        self.assertEqual(self.panorama, ret_val)

    def test_nearest_pandevice_from_device_group_in_pano_dg_fw_ao_chain(self):
        """Runs nearest_pandevice() on the device group.

        This should return the Panorama PanDevice.
        """
        ret_val = self.device_group.nearest_pandevice()

        self.assertEqual(self.panorama, ret_val)

    def test_nearest_pandevice_from_panorama_in_pano_dg_fw_ao_chain(self):
        """Runs nearest_pandevice() on the panorama.

        This should return the Panorama PanDevice.
        """
        ret_val = self.panorama.nearest_pandevice()

        self.assertEqual(self.panorama, ret_val)

    def test_nearest_pandevice_from_firewall_with_no_parents_returns_self(self):
        fw = pandevice.firewall.Firewall(
            'foo', 'bar', 'baz', 'apikey')

        ret_val = fw.nearest_pandevice()

        self.assertEqual(fw, ret_val)


class TestElementStr_7_0(unittest.TestCase):
    """
    [Test section:  element_str()]

    Verify the XML created under various circumstances for various
    objects and setups:

        * HighAvailability with HA1 and HA2 children
        * VirtualRouter with StaticRoute child
        * EthernetInterface
        * Firewall
        * AddressObject
    """

    # 1) HighAvailability with HA1 and HA2 children
    def test_element_str_from_highavailability_with_ha1_and_ha2_children(self):
        expected = b''.join([
            b'<high-availability><enabled>yes</enabled><group><entry name="1">',
            b'<description>my ha conf description</description>',
            b'<configuration-synchronization><enabled>yes</enabled>',
            b'</configuration-synchronization><peer-ip>10.5.1.5</peer-ip>',
            b'<mode><active-passive><passive-link-state>passive state',
            b'</passive-link-state></active-passive></mode>',
            b'<state-synchronization><enabled>no</enabled><ha2-keep-alive>',
            b'<enabled>yes</enabled><action>ha2 do stuff</action><threshold>',
            b'2</threshold></ha2-keep-alive></state-synchronization></entry>',
            b'</group><interface><ha1><ip-address>10.5.1.1</ip-address>',
            b'<netmask>255.255.255.0</netmask><port>ethernet1/6</port>',
            b'<gateway>10.5.1.2</gateway><link-speed>1000</link-speed>',
            b'<link-duplex>auto</link-duplex><monitor-hold-time>7',
            b'</monitor-hold-time></ha1><ha1-backup /><ha2>',
            b'<ip-address>10.6.1.1</ip-address><netmask>255.255.255.0',
            b'</netmask><port>ethernet1/7</port><gateway>10.6.1.2</gateway>',
            b'<link-speed>1000</link-speed><link-duplex>auto</link-duplex>',
            b'</ha2><ha2-backup /><ha3 /></interface></high-availability>',
        ])

        h1o = pandevice.ha.HA1(
            name='ha101', ip_address='10.5.1.1', netmask='255.255.255.0',
            port='ethernet1/6', gateway='10.5.1.2', link_speed='1000',
            link_duplex='auto', monitor_hold_time=7)
        h2o = pandevice.ha.HA2(
            name='ha202', ip_address='10.6.1.1', netmask='255.255.255.0',
            port='ethernet1/7', gateway='10.6.1.2', link_speed='1000',
            link_duplex='auto')
        ha_config = pandevice.ha.HighAvailability(
            'my high availability config', True, '1', 'my ha conf description',
            True, '10.5.1.5', 'active-passive', 'passive state', False, True,
            'ha2 do stuff', 2, 'ethernet1/5')

        ha_config.add(h1o)
        ha_config.add(h2o)

        ret_val = ha_config.element_str()

        self.assertEqual(expected, ret_val)

    # 2) VirtualRouter with StaticRoute child
    def test_element_str_from_virtualrouter_with_sr_parent(self):
        '''StaticRoute > VirtualRouter'''
        expected = b''.join([
            b'<entry name="default"><interface><member>ethernet1/3</member>',
            b'</interface><routing-table><ip><static-route>',
            b'<entry name="my static route"><destination>0.0.0.0/0',
            b'</destination><nexthop><ip-address>192.168.5.1</ip-address>',
            b'</nexthop><interface>ethernet1/4</interface><metric>10</metric>',
            b'</entry></static-route></ip></routing-table></entry>',
        ])

        vro = pandevice.network.VirtualRouter('default', 'ethernet1/3')
        sro = pandevice.network.StaticRoute(
            'my static route', '0.0.0.0/0', 'ip-address',
            '192.168.5.1', 'ethernet1/4')

        vro.add(sro)

        ret_val = vro.element_str()

        self.assertEqual(expected, ret_val)

    # 3) EthernetInterface
    def test_element_str_from_ethernetinterface(self):
        expected = b''.join([
            b'<entry name="ethernet1/1"><layer3><ip><entry name="10.1.1.1" />',
            b'</ip></layer3><link-speed>1000</link-speed><link-duplex>auto',
            b'</link-duplex><link-state>auto</link-state></entry>',
        ])

        o = pandevice.network.EthernetInterface(
            'ethernet1/1', 'layer3', '10.1.1.1', link_speed='1000',
            link_duplex='auto', link_state='auto')

        o_str = o.element_str()
        self.assertEqual(expected, o_str)

    def test_element_str_from_ethernetinterface_in_en_l3s_arp(self):
        '''EthernetInterface > Layer3Subinterface > Arp'''
        expected = b''.join([
            b'<entry name="ethernet1/1"><layer3><ip>',
            b'<entry name="10.3.6.12" /></ip><units>',
            b'<entry name="ethernet1/1.355"><tag>355</tag><ip>',
            b'<entry name="10.20.30.40/24" /></ip><mtu>1500</mtu>',
            b'<adjust-tcp-mss>yes</adjust-tcp-mss><arp>',
            b'<entry name="10.5.10.15"><hw-address>00:30:48:52:cd:dc',
            b'</hw-address></entry></arp></entry></units></layer3></entry>',
        ])

        ao = pandevice.network.Arp('10.5.10.15', '00:30:48:52:cd:dc')
        l3so = pandevice.network.Layer3Subinterface(
            'ethernet1/1.355', 355, '10.20.30.40/24',
            mtu=1500, adjust_tcp_mss=True)
        eio = pandevice.network.EthernetInterface(
            'ethernet1/1', mode='layer3', ip='10.3.6.12')

        l3so.add(ao)
        eio.add(l3so)

        # This is actually a 7.x test
        for o in (ao, l3so, eio):
            o.retrieve_panos_version = mock.Mock(return_value=(7, 0, 0))

        ret_val = eio.element_str()

        self.assertEqual(expected, ret_val)

    def test_element_str_from_ethernetinterface_for_aggregate_group(self):
        expected = b''.join([
            b'<entry name="ethernet1/1"><aggregate-group>ae1',
            b'</aggregate-group></entry>',
        ])
        eio = pandevice.network.EthernetInterface(
            'ethernet1/1', 'aggregate-group', '10.3.6.12',
            aggregate_group='ae1')

        ret_val = eio.element_str()

        self.assertEqual(expected, ret_val)

    # 4) Firewall
    """
    * vsys of the firewall
    * multi_vsys variable is
    * serial is set / not set
    * parent is pano / device group
    """
    def test_element_str_from_firewall_with_pano_parent_and_systemsettings_child(self):
        expected = b''.join([
            b'<entry name="Serial"><vsys>',
            b'<entry name="vsys1" /></vsys></entry>',
        ])

        fw = pandevice.firewall.Firewall(
            'fw1', 'user', 'passwd', 'authkey', serial='Serial', vsys='vsys3')
        pano = pandevice.panorama.Panorama('10.100.5.2')
        conf = pandevice.device.SystemSettings(
            hostname='Hostname-Setting',
            domain='paloaltonetworks.com',
            ip_address='10.20.30.40',
            netmask='255.255.255.0',
            default_gateway='10.20.30.1',
            panorama='10.100.5.2',
            login_banner="This is not the firewall you're looking for..",
            update_server='8.8.8.8',
        )

        fw.add(conf)
        pano.add(fw)

        ret_val = fw.element_str()

        self.assertEqual(expected, ret_val)

    def test_element_str_from_firewall_without_serial_number_raises_error(self):
        fw = pandevice.firewall.Firewall('foo')

        self.assertRaises(
            ValueError,
            fw.element_str)

    def test_element_str_from_firewall_with_dg_pano_parents_and_multi_vsys(self):
        expected = b''.join([
            b'<entry name="serial"><vsys><entry name="vsys3" />',
            b'</vsys></entry>',
        ])

        fw = pandevice.firewall.Firewall(
            'fw1', 'user', 'passwd', 'authkey',
            serial='serial', vsys='vsys3', multi_vsys=True)
        dg = pandevice.panorama.DeviceGroup('my group')
        p = pandevice.panorama.Panorama('pano')

        dg.add(fw)
        p.add(dg)

        ret_val = fw.element_str()

        self.assertEqual(expected, ret_val)

    # 5) AddressObject
    def test_element_str_from_addressobject(self):
        expected = b''.join([
            b'<entry name="webserver"><ip-netmask>192.168.1.100</ip-netmask>',
            b'<description>Intranet web server</description><tag><member>',
            b'https</member><member>http</member></tag></entry>',
        ])
        o = pandevice.objects.AddressObject(
            'webserver', '192.168.1.100', description='Intranet web server',
            tag=['https', 'http'])

        o_str = o.element_str()
        self.assertEqual(expected, o_str)


class TestXpaths_7_0(unittest.TestCase):
    """
    [Test section: xpaths]
    Test that both set and edit xpaths are reported correctly.

    These test cases have various object constructions, and verify that both
    the 'set' and 'edit/delete' xpath can correctly be identified.
    """

    # HA1 / HA2 Tests
    def test_edit_xpath_from_ha1_with_ha_fw_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/deviceconfig/high-availability",
            "/interface/ha1",
        ])

        child = pandevice.ha.HA1()
        parent = pandevice.ha.HighAvailability()
        fw = pandevice.firewall.Firewall()

        parent.add(child)
        fw.add(parent)

        ret_val = child.xpath()

        self.assertEqual(expected, ret_val)

    def test_edit_xpath_from_ha1_with_ha_fw_pano_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/deviceconfig/high-availability",
            "/interface/ha1",
        ])

        child = pandevice.ha.HA1('ha1')
        parent = pandevice.ha.HighAvailability('ha parent')
        fw = pandevice.firewall.Firewall('myfw')
        pano = pandevice.panorama.Panorama('panorama')

        parent.add(child)
        fw.add(parent)
        pano.add(fw)

        ret_val = child.xpath()

        self.assertEqual(expected, ret_val)

    def test_edit_xapth_from_ha2_with_ha_fw_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/deviceconfig/high-availability",
            "/interface/ha2",
        ])

        child = pandevice.ha.HA2('child')
        parent = pandevice.ha.HighAvailability('parent')
        fw = pandevice.firewall.Firewall('myfw')

        parent.add(child)
        fw.add(parent)

        ret_val = child.xpath()

        self.assertEqual(expected, ret_val)

    def test_set_xpath_from_ha2_with_ha_fw_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/deviceconfig/high-availability",
            "/interface",
        ])

        child = pandevice.ha.HA2('child')
        parent = pandevice.ha.HighAvailability('HighAvail')
        fw = pandevice.firewall.Firewall('myfw')

        parent.add(child)
        fw.add(parent)

        ret_val = child.xpath_short()

        self.assertEqual(ret_val, expected)
        self.assertEqual(expected, ret_val)

    # VirtualRouter tests
    def test_edit_xpath_from_virtualrouter_with_sr_fw_parents(self):
        '''Firewall > VirtualRouter > StaticRoute'''
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/routing-table/ip/static-route/entry[@name='sr']",
            "/network/virtual-router/entry[@name='vr']",
        ])

        child = pandevice.network.VirtualRouter('vr')
        parent = pandevice.network.StaticRoute('sr')
        fw = pandevice.firewall.Firewall('fw')
        fw.get_device_version = mock.Mock(return_value=(7, 0, 0))

        parent.add(child)
        fw.add(parent)

        ret_val = child.xpath()

        self.assertEqual(expected, ret_val)

    def test_set_xpath_from_virtualrouter_with_sr_fw_parents(self):
        '''Firewall > VirtualRouter > StaticRoute'''
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/routing-table/ip/static-route/entry[@name='sr']",
            "/network/virtual-router",
        ])

        child = pandevice.network.VirtualRouter('vr')
        parent = pandevice.network.StaticRoute('sr')
        fw = pandevice.firewall.Firewall('fw')
        fw.get_device_version = mock.Mock(return_value=(7, 0, 0))

        parent.add(child)
        fw.add(parent)

        ret_val = child.xpath_short()

        self.assertEqual(expected, ret_val)

    # Arp (EthernetInterface) tests
    def test_edit_xpath_from_arp_with_l3s_ei_fw_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/network/interface/ethernet/entry[@name='Eth Interface Object']",
            "/layer3/units/entry[@name='Layer3 Subint Object']",
            "/arp/entry[@name='arp object']",
        ])

        ao = pandevice.network.Arp('arp object')
        l3so = pandevice.network.Layer3Subinterface('Layer3 Subint Object')
        eio = pandevice.network.EthernetInterface('Eth Interface Object')
        fw = pandevice.firewall.Firewall('fw')
        fw.get_device_version = mock.Mock(return_value=(7, 0, 0))

        l3so.add(ao)
        eio.add(l3so)
        fw.add(eio)

        ret_val = ao.xpath()

        self.assertEqual(expected, ret_val)

    def test_set_xpath_from_arp_with_l3s_ei_fw_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/network/interface/ethernet/entry[@name='Eth Interface Object']",
            "/layer3/units/entry[@name='Layer3 Subint Object']",
            "/arp",
        ])

        ao = pandevice.network.Arp('arp object')
        l3so = pandevice.network.Layer3Subinterface('Layer3 Subint Object')
        eio = pandevice.network.EthernetInterface('Eth Interface Object')
        fw = pandevice.firewall.Firewall('fw')
        fw.get_device_version = mock.Mock(return_value=(7, 0, 0))

        l3so.add(ao)
        eio.add(l3so)
        fw.add(eio)

        ret_val = ao.xpath_short()

        self.assertEqual(expected, ret_val)

    # Firewall tests
    def test_edit_xpath_from_firewall(self):
        # This is not a valid xpath, but its what should happen
        # if there is no parent
        expected = ''.join([
            "/devices/entry[@name='serial']",
        ])

        fw = pandevice.firewall.Firewall(
            'foo', vsys='vsys2', serial='serial')

        ret_val = fw.xpath()

        self.assertEqual(expected, ret_val)

    def test_set_xpath_from_firewall(self):
        # This is not a valid xpath, but its what should happen
        # if there is no parent
        expected = ''.join([
            "/devices",
        ])

        fw = pandevice.firewall.Firewall(
            'foo', vsys='vsys2', serial='serial')

        ret_val = fw.xpath_short()

        self.assertEqual(expected, ret_val)

    def test_edit_xpath_from_firewall_with_pano_parent(self):
        expected = "/config/mgt-config/devices/entry[@name='serial']"

        p = pandevice.panorama.Panorama('pano')
        fw = pandevice.firewall.Firewall(
            'foo', vsys='vsys2', serial='serial')

        p.add(fw)

        ret_val = fw.xpath()

        self.assertEqual(expected, ret_val)

    def test_set_xpath_from_firewall_with_pano_parent(self):
        expected = "/config/mgt-config/devices"

        p = pandevice.panorama.Panorama('pano')
        fw = pandevice.firewall.Firewall(
            'foo', vsys='vsys2', serial='serial')

        p.add(fw)

        ret_val = fw.xpath_short()

        self.assertEqual(expected, ret_val)

    def test_edit_xpath_from_firewall_with_dg_pano_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/device-group/entry[@name='my group']/devices",
            "/entry[@name='serial']",
        ])

        p = pandevice.panorama.Panorama('pano')
        dg = pandevice.panorama.DeviceGroup('my group')
        fw = pandevice.firewall.Firewall(
            'foo', vsys='vsys2', serial='serial')

        dg.add(fw)
        p.add(dg)

        ret_val = fw.xpath()

        self.assertEqual(expected, ret_val)

    def test_set_xpath_from_firewall_with_dg_pano_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/device-group/entry[@name='my group']/devices",
        ])

        p = pandevice.panorama.Panorama('pano')
        dg = pandevice.panorama.DeviceGroup('my group')
        fw = pandevice.firewall.Firewall(
            'foo', vsys='vsys2', serial='serial')

        dg.add(fw)
        p.add(dg)

        ret_val = fw.xpath_short()

        self.assertEqual(expected, ret_val)

    # AddressObject tests
    def test_edit_xpath_from_addressobject_with_fw_parent(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/vsys/entry[@name='vsys2']",
            "/address/entry[@name='ntp server']",
        ])

        ao = pandevice.objects.AddressObject('ntp server')
        fw = pandevice.firewall.Firewall('fw', vsys='vsys2')
        fw.get_device_version = mock.Mock(return_value=(7, 0, 0))

        fw.add(ao)

        ret_val = ao.xpath()

        self.assertEqual(expected, ret_val)

    def test_edit_xpath_from_addressobject_with_dg_panorama_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/device-group/entry[@name='My Group']",
            "/address/entry[@name='webproxy']",
        ])

        ao = pandevice.objects.AddressObject('webproxy')
        dg = pandevice.panorama.DeviceGroup('My Group')
        pano = pandevice.panorama.Panorama('My Panorama')
        pano.get_device_version = mock.Mock(return_value=(7, 0, 0))

        dg.add(ao)
        pano.add(dg)

        ret_val = ao.xpath()

        self.assertEqual(expected, ret_val)

    def test_set_xpath_from_addressobject_with_fw_parent(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/vsys/entry[@name='vsys2']",
            "/address",
        ])

        ao = pandevice.objects.AddressObject('ntp server')
        fw = pandevice.firewall.Firewall('fw', vsys='vsys2')
        fw.get_device_version = mock.Mock(return_value=(7, 0, 0))

        fw.add(ao)

        ret_val = ao.xpath_short()

        self.assertEqual(expected, ret_val)

    def test_set_xpath_from_addressobject_with_dg_panorama_parents(self):
        expected = ''.join([
            "/config/devices/entry[@name='localhost.localdomain']",
            "/device-group/entry[@name='My Group']",
            "/address",
        ])

        ao = pandevice.objects.AddressObject('webproxy')
        dg = pandevice.panorama.DeviceGroup('My Group')
        pano = pandevice.panorama.Panorama('My Panorama')
        pano.get_device_version = mock.Mock(return_value=(7, 0, 0))

        dg.add(ao)
        pano.add(dg)

        ret_val = ao.xpath_short()

        self.assertEqual(expected, ret_val)


class TestVariousSubinterfaceXpaths(unittest.TestCase):
    def test_l2_subinterface_with_firewall_parent(self):
        fw = pandevice.firewall.Firewall('192.168.1.1', 'admin', 'admin', vsys='vsys2')
        iface = pandevice.network.EthernetInterface('ethernet1/3', 'layer2')
        eth = pandevice.network.Layer2Subinterface('ethernet1/3.3', 3)
        iface.add(eth)
        fw.add(iface)

        expected = eth.xpath()

        fw.add(eth)

        self.assertEqual(expected, eth.xpath())

    def test_l2_subinterface_with_vsys_parent(self):
        fw = pandevice.firewall.Firewall('192.168.1.1', 'admin', 'admin')
        vsys = pandevice.device.Vsys('vsys2')
        iface = pandevice.network.EthernetInterface('ethernet1/3', 'layer2')
        eth = pandevice.network.Layer2Subinterface('ethernet1/3.3', 3)
        iface.add(eth)
        vsys.add(iface)
        fw.add(vsys)

        expected = eth.xpath()

        vsys.add(eth)

        self.assertEqual(expected, eth.xpath())

    def test_l3_subinterface_with_firewall_parent(self):
        fw = pandevice.firewall.Firewall('192.168.1.1', 'admin', 'admin', vsys='vsys3')
        iface = pandevice.network.EthernetInterface('ethernet1/4', 'layer3')
        eth = pandevice.network.Layer3Subinterface('ethernet1/4.4', 4)
        iface.add(eth)
        fw.add(iface)

        expected = eth.xpath()

        fw.add(eth)

        self.assertEqual(expected, eth.xpath())

    def test_l3_subinterface_with_vsys_parent(self):
        fw = pandevice.firewall.Firewall('192.168.1.1', 'admin', 'admin')
        vsys = pandevice.device.Vsys('vsys3')
        iface = pandevice.network.EthernetInterface('ethernet1/4', 'layer3')
        eth = pandevice.network.Layer2Subinterface('ethernet1/4.4', 4)
        iface.add(eth)
        vsys.add(iface)
        fw.add(vsys)

        expected = eth.xpath()

        vsys.add(eth)

        self.assertEqual(expected, eth.xpath())


if __name__=='__main__':
    unittest.main()



#!/usr/bin/env python

# Copyright (c) 2014, Palo Alto Networks
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


"""Exception classes used by pandevice package"""

from pan.xapi import PanXapiError


# Exceptions used by PanDevice Class
class PanDeviceError(PanXapiError):
    """Exception for errors in the PanDevice class

    The PanDevice class may raise errors when problems occur such as
    response parsing problems.  This exception class is raised on those
    errors. This class is not for errors connecting to the API, as
    pan.xapi.PanXapiError is responsible for those.

    Attributes:
        message: The error message for the exception
        pan_device: A reference to the PanDevice that generated the exception

    """
    def __init__(self, *args, **kwargs):
        self.pan_device = kwargs.pop('pan_device', None)
        super(PanDeviceError, self).__init__(*args, **kwargs)

class PanDeviceXapiError(PanDeviceError):
    """General error returned by an API call"""
    pass

class PanInvalidCredentials(PanDeviceXapiError):
    pass

class PanURLError(PanDeviceXapiError):
    pass

class PanConnectionTimeout(PanDeviceXapiError):
    pass

class PanJobTimeout(PanDeviceError):
    pass

class PanLockError(PanDeviceError):
    pass

class PanPendingChanges(PanDeviceError):
    pass

class PanCommitInProgress(PanDeviceXapiError):
    pass

class PanInstallInProgress(PanDeviceXapiError):
    pass

class PanCommitFailed(PanDeviceXapiError):
    def __init__(self, *args, **kwargs):
        self.result = kwargs.pop('result', None)
        super(PanCommitFailed, self).__init__("Commit failed", *args, **kwargs)

class PanCommitNotNeeded(PanDeviceXapiError):
    pass

class PanSessionTimedOut(PanDeviceXapiError):
    pass

class PanDeviceNotSet(PanDeviceError):
    pass

class PanNotConnectedOnPanorama(PanDeviceError):
    pass

class PanNotAttachedOnPanorama(PanDeviceError):
    pass

class PanNoSuchNode(PanDeviceXapiError):
    pass

class PanObjectMissing(PanDeviceXapiError):
    pass

class PanHAConfigSyncFailed(PanDeviceXapiError):
    pass

class PanHASyncInProgress(PanDeviceXapiError):
    pass

class PanObjectError(PanDeviceError):
    pass

class PanApiKeyNotSet(PanDeviceError):
    pass

class PanActivateFeatureAuthCodeError(PanDeviceError):
    pass

class PanOutdatedSslError(PanDeviceError):
    pass
